CAAT.Module( {
    defines : "FIAW.Actor.GameOverWindow",
    depends : [
        "CAAT.Foundation.ActorContainer"
    ],
    extendsClass : "CAAT.Foundation.ActorContainer",

    extendsWith : function() {

        return {

            gamePlayScene: null,
            gameOverType: null, //0 Win, 1 tie, 2 wait, 3 disconnected
            director: null,
            gamePlayManager: null,

            replayButton: null,
            menuButton: null,
            infoPanel: null,
            resumeButton: null,

            textReplay: null,

            winOrTieWindow: null,
            cloneBoard: null,
            winOrTieTimer: null,
            timeTransition: 4000,

            menu: null,
            finalWindow: null,
            removeAdsTimer: null,
            countAdsTime: null,
            menuButtonsRef: null,

            removeAdsButton: null,

            __init: function(gamePlayScene, gameOverType){
                this.__super();

                this.gamePlayScene = gamePlayScene;
                this.director = this.gamePlayScene.director;
                this.gamePlayManager = this.gamePlayScene.gamePlayManager;
                this.gameOverType = gameOverType;

                this.setSize(this.gamePlayScene.width, this.gamePlayScene.height)
                    .setGlobalAnchor(0.5, 0.5)
                    .setLocation(this.gamePlayScene.width/2, this.gamePlayScene.height/2);

                this.initContainer();
                return this;
            },

            createWinOrTieWindow: function(){
                var self = this;

                this.winOrTieWindow = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage("fondo-intermedia1"))
                    .rescale()
                    .enableEvents(false)
                    .setAlpha(0)
                    .setGlobalAnchor(0.5, 0.5)
                    .setLocation(this.width/2, this.height/2);

                var behavior = this.fadeInBheavior();

                behavior.addListener({
                    behaviorExpired: function(behavior, time, actor){
                        actor.enableEvents(true);
                    }
                });

                this.winOrTieWindow.addBehavior(behavior);

                this.winOrTieTimer = this.gamePlayScene.createTimer(
                    this.gamePlayScene.time,
                    this.timeTransition,
                    function() {
                        self.__closeWinOrTie(null, self);
                });

                this.winOrTieWindow.mouseDown = function(e){
                    if(self.winOrTieTimer) {
                        self.winOrTieTimer.cancel();
                        self.winOrTieTimer = null;
                    }
                    self.__closeWinOrTie(e, self);
                };

                this.addChild(this.winOrTieWindow);

                //Win Type
                if(this.gameOverType === 0){
                    this.cloneBoard = this.cloneGameBoard();

                    //Muestro las fichas ganadoras
                    var bgTurnChip = [this.director.getImage('fichaamarilla'), this.director.getImage('ficharoja')],
                        arrayWin = this.gamePlayManager.arrayWin;

                    var exampleChip = null;

                    for(var i = 0; i < arrayWin.length; i++){
                        var coord = this.getChipLocation(arrayWin[i][0],arrayWin[i][1]);

                        var chipWin = new FIAW.Actor.PlayerChip()
                            .setBackgroundImage(bgTurnChip[this.gamePlayScene.turnGame])
                            .rescale()
                            .setAlpha(1)
                            .setLocation(coord.x, coord.y);

                        this.cloneBoard.addChild(chipWin);

                        //Almaceno una ficha fuera del scope para consultar valores de tamaño
                        if(!exampleChip) exampleChip = chipWin;

                    }

                    this.showTextAndBar(exampleChip, arrayWin);

                    this.addChild(this.cloneBoard);
                }else{
                    //Tie Type
                    var tieText = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(FIAW.localeText.getString("T_TIE_GAME", "TIE GAME"))
                        .setBaseline("middle")
                        .setTextAlign("center")
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setScale(this.gamePlayScene.gameScale, this.gamePlayScene.gameScale)
                        .setLocation(this.winOrTieWindow.width/2, this.winOrTieWindow.height/2);

                    this.winOrTieWindow.addChild(tieText);

                }

            },

            getPlayerName: function(turn){
                var name;
                if(turn === 0){
                    name = this.gamePlayScene.playerOneText.text;
                }else{
                    name = this.gamePlayScene.playerTwoText.text;
                }

                return name;
            },

            __closeWinOrTie: function(e, self){
                self.openFinalWindow();
                self.winOrTieWindow.setDiscardable(true).setExpired(true);
                if(this.cloneBoard) self.cloneBoard.setDiscardable(true).setExpired(true);
            },

            openFinalWindow: function(){
                var self = this;

                this.finalWindow = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage("fondo-intermedia2"))
                    .rescale()
                    .enableEvents(true)
                    .setAlpha(1)
                    .setGlobalAnchor(0.5, 0.5)
                    .setLocation(this.width/2, this.height/2);

                this.menu = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0.5)
                    .setScale(this.gamePlayScene.gameScale, this.gamePlayScene.gameScale)
                    .enableEvents(true)
                    .setLocation(this.finalWindow.width/2, this.finalWindow.height/2);

                var playerOne = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('player-one'))
                    .setGlobalAnchor(0, 0)
                    .rescale()
                    .setLocation(this.menu.width/2, 0);

                var playerTwo= new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('player-two'))
                    .setGlobalAnchor(0, 0)
                    .rescale()
                    .setLocation(this.menu.width/2, playerOne.height + 10);

                var offSetX = 60;

                var playerName = [
                    new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(this.getPlayerName(FIAW.Globals.G_PLAYER_1))
                        .setBaseline("middle")
                        .setTextAlign('left')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(offSetX, playerOne.height/2 - 10),
                    new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(this.getPlayerName(FIAW.Globals.G_PLAYER_2))
                        .setBaseline("middle")
                        .setTextAlign('left')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(offSetX, playerTwo.height/2 - 10)
                ];

                var playerWins = [
                    new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(this.gamePlayScene.getCountWinString(FIAW.Globals.G_PLAYER_1))
                        .setBaseline("middle")
                        .setTextAlign('right')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(playerOne.width - offSetX, playerOne.height/2 - 10),
                    new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(this.gamePlayScene.getCountWinString(FIAW.Globals.G_PLAYER_2))
                        .setBaseline("middle")
                        .setTextAlign('right')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(playerTwo.width - offSetX, playerTwo.height/2 - 10)
                ];

                playerOne.addChild(playerName[0]);
                playerTwo.addChild(playerName[1]);
                playerOne.addChild(playerWins[0]);
                playerTwo.addChild(playerWins[1]);

                this.menu.addChild(playerOne);
                this.menu.addChild(playerTwo);

                var playAgainButton = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('play-again'))
                    .setGlobalAnchor(0, 0)
                    .rescale()
                    .enableEvents(true)
                    .setLocation(this.menu.width/2, playerTwo.y + playerTwo.height + 10);

                playAgainButton.addChild(
                    new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(FIAW.localeText.getString("T_PLAY_AGAIN", "PLAY AGAIN"))
                        .setBaseline("middle")
                        .setTextAlign('center')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(playAgainButton.width/2, playAgainButton.height/2 - 10)
                );

                var gotoMenuButton = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('play-again'))
                    .setGlobalAnchor(0, 0)
                    .rescale()
                    .enableEvents(true)
                    .setLocation(this.menu.width/2, playAgainButton.y + playAgainButton.height + 10);

                gotoMenuButton.addChild(
                    new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap-white"))
                        .setText(FIAW.localeText.getString("T_GOTO_MENU", "GO TO MENU"))
                        .setBaseline("middle")
                        .setTextAlign('center')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(gotoMenuButton.width/2, gotoMenuButton.height/2 - 10)
                );

                gotoMenuButton.mouseDown = function(e){
                    if(self.gamePlayScene.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE){
                        self.gamePlayScene.gamePlayManager.multiplayerManager.setDisconnect(true);
                    }
                    self.gamePlayManager.returnMenuScene();
                    self.setDiscardable(true).setExpired(true);
                };

                playAgainButton.mouseDown = function(e){
                    self.__replayButtonPressed(e, self);
                };

                var ww, hh;

                if(FIAW.adsEnabled){
                    this.removeAdsButton = new CAAT.Foundation.ActorContainer()
                        .setBackgroundImage(this.director.getImage('remove'))
                        .setGlobalAnchor(0, 0)
                        .rescale()
                        .enableEvents(true)
                        .setLocation(this.menu.width/2, gotoMenuButton.y + gotoMenuButton.height + 10);

                    this.removeAdsButton.addChild(
                        new CAAT.Foundation.UI.TextActor()
                            .setFont(FIAW.Utils.getFont("sixcap"))
                            .setText(FIAW.localeText.getString("T_REMOVE_ADS", "REMOVE ADS"))
                            .setBaseline("middle")
                            .setTextAlign('center')
                            .cacheAsBitmap()
                            .enableEvents(false)
                            .setLocation(this.removeAdsButton.width/2, this.removeAdsButton.height/2 - 10)
                    );

                    this.removeAdsButton.mouseDown = function(e){
                        FIAW.Utils.inAppsManager.removeAds();
                    };

                    this.menu.addChild(this.removeAdsButton);

                    hh = 5 * (playerOne.height + 10);

                    if(this.gamePlayScene.gameMode != FIAW.Globals.G_PLAYER_VS_ONLINE){
                        this.initAdsTimer([playAgainButton, gotoMenuButton, this.removeAdsButton]);
                    }

                }else{
                    hh = 4 * (playerOne.height + 10);
                }
                ww = playerOne.width;

                this.menu.addChild(playAgainButton);
                this.menu.addChild(gotoMenuButton);

                this.menu.setSize(ww, hh);

                this.finalWindow.addChild(this.menu);

                this.addChild(this.finalWindow);

            },

            initAdsTimer: function(buttons){
                var self = this;
                this.menuButtonsRef = buttons;

                for(var i = 0; i < buttons.length; i++){
                    buttons[i].setVisible(false).enableEvents(false);
                }

                this.countAdsTime = new CAAT.Foundation.ActorContainer()
                        .setBackgroundImage(this.director.getImage('remove'))
                        .setGlobalAnchor(0, 0)
                        .rescale()
                        .enableEvents(false)
                        .setLocation(buttons[0].x, buttons[0].y);

                var countText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap"))
                    .setText((FIAW.Globals.G_ADS_TIME/1000).toString())
                    .setBaseline("middle")
                    .setTextAlign('center')
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(this.countAdsTime.width/2, this.countAdsTime.height/2 - 10);

                this.countAdsTime.addChild(countText);
                this.menu.addChild(this.countAdsTime);

                this.removeAdsTimer = this.gamePlayScene.createTimer(
                    this.gamePlayScene.time,
                    FIAW.Globals.G_ADS_TIME,
                    function(){
                        self.showMenuButtons();
                    },
                    function(scene_time, timer_time, timertask_instance){
                        var t = Math.floor((FIAW.Globals.G_ADS_TIME - timer_time)/1000);
                        countText.setText(t.toString());
                    }
                );

                if(this.gamePlayScene.gameMode != FIAW.Globals.G_PLAYER_VS_ONLINE){
                    FIAW.Utils.adsManager.showFullScreen(this);
                }
            },

            stopAdsTimer: function(){
                if(this.removeAdsTimer){
                    this.removeAdsTimer.cancel();
                    this.removeAdsTimer = null;
                }
                this.showMenuButtons();
            },

            showMenuButtons: function(){
                this.countAdsTime.setDiscardable(true).setExpired(true);
                for(var i = 0; i < this.menuButtonsRef.length; i++){
                    this.menuButtonsRef[i].setVisible(true).enableEvents(true);
                }
            },

            //Calculo la rotación y posición del texto y la barra a mostrar.
            showTextAndBar: function(exampleChip, arrayWin){
                var chipCoord;
                var bar = {
                    x : null, y : null
                };
                var letter = {
                    x : null, y : null, align : null
                };

                var winBar = new CAAT.Foundation.Actor()
                    .setBackgroundImage(this.director.getImage('raya-intermedia1'))
                    .rescale()
                    .enableEvents(false);

                var winText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap"))
                    .setText(this.getPlayerName(this.gamePlayScene.turnGame))
                    .setBaseline("top")
                    .cacheAsBitmap()
                    .enableEvents(false);

                var winText2 = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap"))
                    .setText(FIAW.localeText.getString("T_WIN", "WIN"))
                    .setBaseline("top")
                    .cacheAsBitmap()
                    .enableEvents(false);


                //Siempre se usa la primera ficha del array como referencia porque es la que está más a la derecha
                switch(this.gamePlayManager.winType){
                    case 'vertical':
                        chipCoord = this.getChipLocation(arrayWin[0][0],arrayWin[0][1]);

                        bar.y = chipCoord.y - exampleChip.height/4;
                        letter.y = bar.y;
    
                        if(arrayWin[0][0] <= 3){
                            bar.x = chipCoord.x + exampleChip.width - exampleChip.width/4;
                            letter.align = "left";
                            letter.x = bar.x + 10;
                        }else{
                            bar.x = chipCoord.x - exampleChip.width + exampleChip.width/4;
                            letter.align = "right";
                            letter.x = bar.x - 10;
                        }

                        winBar.setLocation(bar.x, bar.y);
                        winText.setLocation(letter.x, letter.y - 40).setTextAlign(letter.align);
                        winText2.setLocation(letter.x, letter.y + winText.height - 80).setTextAlign(letter.align);
                    break;

                    case 'horizontal':
                        chipCoord = this.getChipLocation(arrayWin[0][0],arrayWin[0][1]);

                        bar.x = chipCoord.x - exampleChip.width/4;
                        if(arrayWin[0][0] <= 2){
                            letter.x = bar.x;
                            letter.align = "left";
                        }else{
                            letter.x = bar.x + winBar.height - 10;
                            letter.align = "right";
                        }

                        if(arrayWin[0][1] < 3){ //Por debajo
                            bar.y = chipCoord.y + exampleChip.height - exampleChip.height/4;
                            letter.y = bar.y;
                        }else{ //por encima
                            bar.y = chipCoord.y - exampleChip.height + exampleChip.height/4;
                            letter.y = bar.y - winText.height*2 + 80;
                        }

                        winBar.setLocation(bar.x, bar.y).setRotationAnchored(-90 * Math.PI / 180, 0, 0);
                        winText.setLocation(letter.x, letter.y - 40).setTextAlign(letter.align);
                        winText2.setLocation(letter.x, letter.y + winText.height - 80).setTextAlign(letter.align);
                    break;

                    case 'diag-top-down':
                        chipCoord = this.getChipLocation(arrayWin[0][0],arrayWin[0][1]);

                        if(arrayWin[0][0] <= 2){
                            bar.x = chipCoord.x + exampleChip.width;
                            letter.x = bar.x + winText.width + 20;
                            letter.align = "left";
                            bar.y = chipCoord.y;
                            letter.y = bar.y - 60;
                        }else{
                            bar.x = chipCoord.x - exampleChip.width/2;
                            letter.x = bar.x + winText.width - winText.width/3;
                            letter.align = "right";
                            bar.y = chipCoord.y + exampleChip.width;
                            letter.y = bar.y + winBar.height/3;
                        }

                        winBar.setLocation(bar.x, bar.y).setRotationAnchored(-45 * Math.PI / 180, 0, 0);
                        winText.setLocation(letter.x, letter.y - 40).setTextAlign(letter.align);
                        winText2.setLocation(letter.x, letter.y + winText.height - 80).setTextAlign(letter.align);
                    break;

                    case 'diag-down-top':
                        //Solo en este caso usamos la ultima ficha como referencia
                        chipCoord = this.getChipLocation(arrayWin[3][0],arrayWin[3][1]);

                        if(arrayWin[0][0] <= 2){
                            bar.x = chipCoord.x + exampleChip.width/4;
                            letter.x = bar.x - winText.width + winText.width/3;
                            letter.align = "left";
                            bar.y = chipCoord.y + exampleChip.width;
                            letter.y = bar.y + winBar.height/3;
                        }else{
                            bar.x = chipCoord.x - exampleChip.width - exampleChip.width/2;
                            letter.x = bar.x - winText.width - 30;
                            letter.align = "right";
                            bar.y = chipCoord.y;
                            letter.y = bar.y - 60;
                        }

                        winBar.setLocation(bar.x, bar.y).setRotationAnchored(45 * Math.PI / 180, 0, 0);
                        winText.setLocation(letter.x, letter.y - 40).setTextAlign(letter.align);
                        winText2.setLocation(letter.x, letter.y + winText.height - 80).setTextAlign(letter.align);
                    break;
                }

                this.cloneBoard.addChild(winBar);
                this.cloneBoard.addChild(winText);
                this.cloneBoard.addChild(winText2);
            },

            getChipLocation: function(col, row){
                 var xx = parseFloat(this.gamePlayScene.coordsCols[col].toFixed(3)),
                    yy = (this.gamePlayScene.gameBoard.height - this.gamePlayScene.tileSizeH/2) - (((this.gamePlayScene.boardColsMap[col].length-1) - row) * this.gamePlayScene.tileSizeH);

                return { x : xx, y : yy };
            },

            cloneGameBoard: function(){

                var gameBoard = this.gamePlayScene.gameBoard;

                var clone = new CAAT.Foundation.ActorContainer()
                    .setSize(gameBoard.width, gameBoard.height)
                    .setGlobalAnchor(0.5, 0.5)
                    .enableEvents(false)
                    .setScale(gameBoard.scaleX, gameBoard.scaleY)
                    .setLocation(gameBoard.x, gameBoard.y);

                return clone;

            },

            fadeInBheavior: function(duration){

                duration = duration || 500;

                var behavior = new CAAT.Behavior.AlphaBehavior()
                    .setValues(0, 1)
                    .setDelayTime(0, duration)
                    .setCycle(false);

                return behavior;
            },

            initContainer: function(){

                if(this.gameOverType === 0 || this.gameOverType === 1){
                    this.createWinOrTieWindow();
                }else{
                    this.createWaitOrDisconnectWindow();
                }
            },

            createWaitOrDisconnectWindow: function(){
                var self = this;

                this.finalWindow = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage("fondo-intermedia2"))
                    .rescale()
                    .enableEvents(true)
                    .setAlpha(1)
                    .setGlobalAnchor(0.5, 0.5)
                    .setLocation(this.width/2, this.height/2);

                this.addChild(this.finalWindow);

                var msgBox = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0.5)
                    .setSize(50, 50)
                    .setScale(this.gamePlayScene.gameScale, this.gamePlayScene.gameScale)
                    .enableEvents(false)
                    .setLocation(this.finalWindow.width/2, this.finalWindow.height/2);

                this.finalWindow.addChild(msgBox);

                var text;
                if(this.gameOverType == 2){
                    text = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(FIAW.localeText.getString("T_WAIT_OPPONENT", "WAITING OPPONENT"))
                        .setBaseline("middle")
                        .setTextAlign('center')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(msgBox.width/2, msgBox.height/2 - 10);
                }else{
                    text = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(FIAW.localeText.getString("T_DISCONNECT_OPPONENT", "OPPONENT DISCONNECTED"))
                        .setBaseline("middle")
                        .setTextAlign('center')
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(msgBox.width/2, msgBox.height/2 - 10);

                    //Si es una desconexión al hacer tap volvemos al menú
                    this.finalWindow.mouseDown = function(e){
                        self.gamePlayManager.returnMenuScene();
                        self.setDiscardable(true).setExpired(true);
                    };
                }

                msgBox.addChild(text);
            },

            __replayButtonPressed: function(e, self){
                self.gamePlayScene.resetGame();
                
                if(self.gamePlayScene.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE){
                    var multiplayerManager = self.gamePlayManager.multiplayerManager;

                    var playerID = -1;
                    for(var i = 0; i < multiplayerManager.players.length; i++){
                        if(multiplayerManager.players[i].match){
                            playerID = i;
                        }
                    }

                    if(multiplayerManager.playAgainPlayerCount === 0) multiplayerManager.waitForPlayerWindow();
                    multiplayerManager.players[playerID].match.sendDataToAllPlayers(JSON.stringify(['playAgain']));

                }else{
                    self.gamePlayScene.__initialize(true);
                }
            }

        };

    }

});