
CAAT.Module( {
    defines : "FIAW.Actor.GamePlayScene",
    depends : [
        "CAAT.Foundation.Scene",
        "FIAW.Actor.PlayerChip",
        "FIAW.Actor.TimeClock",
        "FIAW.Actor.GameOverWindow",
        "FIAW.Actor.PauseWindow"
    ],
    extendsClass : "CAAT.Foundation.Scene",

    extendsWith : function() {

        return {

            defaultGameMode: FIAW.Globals.G_PLAYER_VS_PLAYER,
            gameMode: null,
            director: null,
            gameScale: null,
            gamePlayManager: null,

            backgroundContainer: null,
            gameBackgroundImage: null,
            gameContainer: null,
            topHud: null,
            gameBoardSpace: null,
            gameBoard: null,
            adsSpace: null,

            turnGame: null,
            turnTime: null,
            turnChip: [null, null],
            actualChip: null,
            blockTurn: false,
            botTurn: null,
            tunrStart: FIAW.Globals.G_PLAYER_1, //Quien comenzó el juego

            timer: null,
            avatarPlayer: [null, null],
            pauseButton: null,

            boardColsMap: null,

            coordsCols: null,
            tileSize: null,
            tileSizeH: null,

            pause: false,
            pauseRemainingTime: null,
            pauseWin: null,
            gameOverWindow: null,

            showGuideCoord: null,
            showGuideContainer: null,

            timeCache: null,

            textWin: [null, null],
            countWin: [0, 0],

            playerOneText: null,
            playerTwoText: null,

            //Datos concernientes al modo multjugador online
            multiplayerData: {
                firstTurn : null,
                players : [null, null],
                playersName : [null,null],
                localID : null
            },

            transBackground : null,

            arrowTurn : [],

            __init: function(gamePlayManager){
                this.__super();

                this.setFillStyle('#000');
                this.gamePlayManager = gamePlayManager;
                this.director = this.gamePlayManager.director;
                this.initBackgroundContainer();
                this.initGameContainer();

                return this;
            },

            activated: function(){

                this.__initialize();
                return this;
            },

            initBackgroundContainer: function(){

                this.backgroundContainer = new CAAT.Foundation.ActorContainer()
                    .setBounds(0, 0, this.director.width, this.director.height);

                this.addChild(this.backgroundContainer);

                this.gameBackgroundImage= new CAAT.Actor()
                    .setBackgroundImage( this.director.getImage('fondo'), true)
                    .enableEvents(false)
                    .setSize(this.director.width, this.director.height)
                    .setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE)
                    .setLocation(this.director.width/2,this.director.height/2)
                    .setGlobalAnchor(0.5,0.5);

                this.backgroundContainer.addChild(this.gameBackgroundImage);

            },

            /* Contenedor principal y adaptación del background */
            initGameContainer: function(){

                var moreHeightForBoard = 500;

                this.gameScale = Math.min(this.director.width/this.director.getValueForKey("W"), this.director.height/(this.director.getValueForKey("H")+moreHeightForBoard));

                this.gameContainer = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0)
                    .setScale(this.gameScale, this.gameScale)
                    .setBounds(
                        this.director.width/2,
                        0,
                        this.director.getValueForKey("W"),
                        this.director.getValueForKey("H")
                    )
                    .setLocation(this.director.width/2, 0)
                    //.setFillStyle('#9f0')
                    .setAlpha(0.5);

                this.addChild(this.gameContainer);

                if(!navigator.isCocoonJS){
                    var menuScale = Math.min(this.director.width/this.director.getValueForKey("W"), this.director.height/(this.director.getValueForKey("H")));
                    this.gameBackgroundImage.setSize(this.gameContainer.width*menuScale,this.gameBackgroundImage.height);
                }

            },

            __initialize: function(changeTurn){

                if(this.gameOverWindow) this.removeChild(this.gameOverWindow);
                
                changeTurn = changeTurn || false;

                if(!changeTurn && this.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE) {
                    this.turnGame = this.multiplayerData.firstTurn;
                    this.turnStart = this.turnGame;
                }else if(!changeTurn){
                    this.turnGame = FIAW.Globals.G_PLAYER_1;
                    this.turnStart = FIAW.Globals.G_PLAYER_1;
                }else{
                    this.turnStart = (this.turnStart == FIAW.Globals.G_PLAYER_1) ? FIAW.Globals.G_PLAYER_2 : FIAW.Globals.G_PLAYER_1;
                    this.turnGame = this.turnStart;
                }

                this.turnTime = FIAW.Globals.G_TURN_TIME;
                this.turnChip = [this.director.getImage('fichaamarilla'), this.director.getImage('ficharoja')];
                this.arrowTurn = [];
                this.botTurn = (this.gameMode == FIAW.Globals.G_PLAYER_VS_COMPUTER) ? FIAW.Globals.G_PLAYER_2 : null;
                this.initTopHud();
                this.initGameBoard();

                //Añadimos 1 al contador de rate siempre que no sea el modo online
                if(this.gameMode != FIAW.Globals.G_PLAYER_VS_ONLINE){
                    FIAW.Utils.rate.setPlayedGames(FIAW.Utils.rate.getPlayedGames()+1);
                }else{

                    /* Compruebo nuevamente si la musica está activada porque si entran en 
                    modo multijugador, al salir de google play games y entrar en la escena la musica
                    deja de sonar */
                    if(FIAW.musicEnabled) this.director.musicPlay('music');

                    console.log(this.multiplayerData.playersName);

                }

                this.showTurn();

            },

            resetGame: function(){
                this.gameBoard.emptyChildren();
                this.gameContainer.removeChild(this.topHud);
                this.removeChild(this.gameBoardSpace);
                this.removeChild(this.gameBoard);
                this.stopTime();
                this.pauseRemainingTime = null;
                this.pause = false;
                this.blockTurn = false;
            },

            __pauseButtonPressed: function(e, self){

                if(self.pause){
                    self.pause = false;
                    this.gameBoard.enableEvents(true);
                    if(this.pauseRemainingTime){
                        this.startTime();
                    }
                }else{
                    self.pause = true;
                    self.stopTime();
                    self.gameBoard.enableEvents(false);
                }

                this.pauseWin.menu();
            },

            initTopHud: function(){
                this.topHud = new CAAT.Foundation.ActorContainer()
                    .setSize(this.gameContainer.width, 400)
                    .setGlobalAnchor(0.5, 0)
                    //.setFillStyle('#fb0')
                    .setAlpha(0.5)
                    .setLocation(this.gameContainer.width/2, 0);

                this.pauseButton = new FIAW.Actor.TimeClock(this.director)
                    .setLocation(this.gameContainer.width/2, this.topHud.height/2)
                    .enableEvents(true);

                var playerOneName = FIAW.Utils.getCorrectName(FIAW.localeText.getString("T_PLAYER_ONE", "PLAYER ONE"));
                var playerTwoName = FIAW.Utils.getCorrectName(FIAW.localeText.getString("T_PLAYER_TWO", "PLAYER TWO"));
                
                if(this.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE) {
                    this.pauseButton.enableEvents(false).pauseIcon.setAlpha(0);

                    if(this.multiplayerData.playersName[FIAW.Globals.G_PLAYER_1]) playerOneName = FIAW.Utils.getCorrectName(this.multiplayerData.playersName[FIAW.Globals.G_PLAYER_1]);
                    if(this.multiplayerData.playersName[FIAW.Globals.G_PLAYER_2]) playerTwoName = FIAW.Utils.getCorrectName(this.multiplayerData.playersName[FIAW.Globals.G_PLAYER_2]);
                }

                var self = this;
                this.pauseButton.mouseDown = function(e){
                    self.__pauseButtonPressed(e, self);
                };

                this.topHud.addChild(this.pauseButton);

                this.playerOneText = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(playerOneName + " ")
                        .setTextAlign("left")
                        .setBaseline("middle")
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(70, this.pauseButton.y);

                this.topHud.addChild(this.playerOneText);

                this.avatarPlayer[FIAW.Globals.G_PLAYER_1] = new CAAT.Foundation.Actor()
                    .setBackgroundImage(this.director.getImage('jugador1'))
                    .setGlobalAnchor(0, 0.5)
                    .rescale()
                    .setLocation(40, this.topHud.height/2 + 105);

                this.topHud.addChild(this.avatarPlayer[FIAW.Globals.G_PLAYER_1]);

                this.avatarPlayer[FIAW.Globals.G_PLAYER_2] = new CAAT.Foundation.Actor()
                    .setBackgroundImage(this.director.getImage('jugador2'))
                    .setGlobalAnchor(0, 0.5)
                    .rescale()
                    .setLocation(this.topHud.width - 20 - 40, this.topHud.height/2 + 105);

                this.topHud.addChild(this.avatarPlayer[FIAW.Globals.G_PLAYER_2]);

                this.playerTwoText = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(playerTwoName + " ")
                        .setTextAlign("right")
                        .setBaseline("middle")
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(this.topHud.width - 20 - 40 - 20, this.pauseButton.y);

                this.topHud.addChild(this.playerTwoText);

                this.textWin[FIAW.Globals.G_PLAYER_1] = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap"))
                    .setText(this.getCountWinString(FIAW.Globals.G_PLAYER_1))
                    .setTextAlign("left")
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(this.playerOneText.x + this.playerOneText.width + 30, this.pauseButton.y);

                this.textWin[FIAW.Globals.G_PLAYER_2] = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap"))
                    .setText(this.getCountWinString(FIAW.Globals.G_PLAYER_2))
                    .setTextAlign("right")
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(this.playerTwoText.x - 5, this.pauseButton.y);

                this.topHud.addChild(this.textWin[FIAW.Globals.G_PLAYER_1]);
                this.topHud.addChild(this.textWin[FIAW.Globals.G_PLAYER_2]);
                this.gameContainer.addChild(this.topHud);

                this.pauseWin = new FIAW.Actor.PauseWindow(this);
                this.gameContainer.addChild(this.pauseWin);

                this.arrowTurn.push(
                    new CAAT.Foundation.Actor()
                        .setBackgroundImage(this.director.getImage('flechitaturno'))
                        .setGlobalAnchor(1, 0.5)
                        .rescale()
                        .setAlpha(0)
                        .setLocation(this.pauseButton.x - this.pauseButton.width/2 -10, this.pauseButton.y)
                );

                this.arrowTurn.push(
                    new CAAT.Foundation.Actor()
                        .setBackgroundImage(this.director.getImage('flechitaturno'))
                        .setGlobalAnchor(0, 0.5)
                        .rescale()
                        .setAlpha(0)
                        .rotateTo(180 * (Math.PI/180), 0, 0, 0.5, 0.5)
                        .setLocation(this.pauseButton.x +this.pauseButton.width/2 +8, this.pauseButton.y -2)
                );

                this.topHud.addChild(this.arrowTurn[0]);
                this.topHud.addChild(this.arrowTurn[1]);

                this.startTime();
            },

            showTurn: function(){
                var turnActor = (this.turnGame == 1) ? this.playerTwoText : this.playerOneText,
                    otherActor = (this.turnGame == 1) ? this.playerOneText : this.playerTwoText;

                turnActor.setAlpha(1);
                otherActor.setAlpha(0.40);

                this.textWin[0].setAlpha(0.40);
                this.textWin[1].setAlpha(0.40);
                this.textWin[this.turnGame].setAlpha(1);

                this.arrowTurn[0].setAlpha(0);
                this.arrowTurn[1].setAlpha(0);
                this.arrowTurn[this.turnGame].setAlpha(1);

                var behavior_turn = new CAAT.Behavior.ScaleBehavior()
                    .setValues(1, 1.1, 1, 1.1, 0.5, 0.5)
                    .setDelayTime(0, 300)
                    .setPingPong()
                    .setCycle(false);

                turnActor.addBehavior(behavior_turn);

            },

            getCountWinString: function(player){
                var r;
                if(this.countWin[player] < 10){
                    r = "0" + this.countWin[player].toString();
                }else{
                    r = this.countWin[player].toString();
                }

                return r + " ";
            },

            stopTime: function(){
                if(this.timer) {
                    this.timer.cancel();
                    this.timer = null;
                }
            },

            startTime: function(){
                this.stopTime();
                this.blockTurn = false;

                if(this.pauseRemainingTime){
                    duration = this.pauseRemainingTime;
                    this.pauseRemainingTime = null;
                }else{
                    duration = FIAW.Globals.G_TURN_TIME * 1000;
                }

                var self = this;
                var botMove = false;
                var initBot = false;

                if(this.gameMode == FIAW.Globals.G_PLAYER_VS_COMPUTER && this.turnGame == this.botTurn){
                    this.blockTurn = true;
                    botMove = true;
                }

                this.timer = this.createTimer(
                    this.time,
                    duration,
                    function(scene_time, timer_time, timertask_instance){
                        //Si se acaba el tiempo y el modo es multijugador solo tira al azar el jugador que juega actualmente
                        if(self.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE){
                            if(self.gamePlayManager.multiplayerManager.isLocalTurn(self.turnGame)){
                                self.gamePlayManager.multiplayerManager.timeOutMove();
                            }
                        }else{

                            //En cualquier otro modo la ficha cae al azar directamente.
                            self.gamePlayManager.botManager.timeOutMove();
                        }
                    },
                    function(scene_time, timer_time, timertask_instance){

                        if(botMove && !initBot && duration - timer_time <= (FIAW.Globals.G_TURN_TIME*1000) - (FIAW.Globals.G_BOT_TURN_TIME*1000)){
                            self.gamePlayManager.botManager.analizeAndMove();
                            initBot = true;

                        }

                        if(!self.pause){

                            var floatTime = (parseFloat(duration-timer_time).toFixed(1));

                            if(floatTime != self.timeCache){
                                self.timeCache = floatTime;
                                self.pauseButton.setTime(floatTime);
                            }

                        }
                    },
                    function(scene_time, timer_time, timertask_instance){
                        self.pauseRemainingTime = duration - timer_time;
                    }
                );

            },

            initGameBoard: function(){

                this.gameBoardSpace = new CAAT.Foundation.ActorContainer()
                .setBackgroundImage(this.director.getImage('casillero'))
                    //.setSize(this.gameContainer.width, 1500)
                    .setGlobalAnchor(0.5, 0.5)
                    .setFillStyle('#5f1')
                    .setAlpha(0.5)
                    .rescale()
                    .setScale(this.gameScale, this.gameScale)
                    .setLocation(this.director.width/2, this.director.height/2);

                if(this.gameBoardSpace.y - (this.gameBoardSpace.height*this.gameScale)/2 < this.topHud.height*this.gameScale){
                    this.gameBoardSpace.setLocation(this.director.width/2, this.topHud.height*this.gameScale + (this.gameBoardSpace.height*this.gameScale)/2);
                }

                this.addChild(this.gameBoardSpace);

                var rows = FIAW.Globals.G_BOARD_ROWS,
                    cols = FIAW.Globals.G_BOARD_COLS;

                //this.tileSize = Math.min(this.gameBoardSpace.width/cols, this.gameBoardSpace.height/rows);
                this.tileSize = this.gameBoardSpace.width/cols;
                this.tileSizeH = this.gameBoardSpace.height/rows;
                this.coordsCols = [];
                this.boardColsMap = [];
                for(var i = 0; i < FIAW.Globals.G_BOARD_COLS; i++){
                    var coord = (this.tileSize/2 + this.tileSize*i);
                    this.coordsCols.push(parseFloat((coord).toFixed(3)));

                    var rowState = [];
                    for(var n = 0; n < FIAW.Globals.G_BOARD_ROWS; n++){
                        rowState.push(false);
                    }

                    this.boardColsMap.push(rowState);
                }

                this.gameBoard = new CAAT.Foundation.ActorContainer()
                    //.setBackgroundImage(this.director.getImage('casillero'))
                    //.setSize(this.tileSize * cols, this.tileSize * rows)
                    .setSize(this.gameBoardSpace.width, this.gameBoardSpace.height)
                    .setGlobalAnchor(0.5, 0.5)
                    //.setFillStyle('red')
                    .setAlpha(0.7)
                    .enableEvents(true)
                    .setScale(this.gameScale, this.gameScale)
                    .setLocation(this.gameBoardSpace.x, this.gameBoardSpace.y);

                var self = this;
                this.gameBoard.mouseDown = function(e){
                    if(self.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE && !self.gamePlayManager.multiplayerManager.isLocalTurn(self.turnGame)) self.blockTurn = true;

                    if(!self.blockTurn && !self.actualChip){
                        self.__createChip(e, self);
                        self.director.audioPlay('snd_create_chip');
                        self.showGuide(e.x);
                    }
                };

                this.gameBoard.mouseDrag = function(e){
                    if(!self.blockTurn && self.actualChip){
                        if((e.x > self.tileSize/2)&&(e.x < self.gameBoard.width - self.tileSize/2)){
                            self.actualChip.x = e.x;
                            self.showGuide(e.x);
                        }
                    }
                };

                this.gameBoard.mouseUp = function(e){
                    if(!self.blockTurn && self.actualChip) {
                        self.__dropChip(e, self);

                    }
                };


                this.addChild(this.gameBoard);

            },

            __calculateCol: function(x, self){
                var xx = x,
                    calc = 9999, i;

                for(i in self.coordsCols){
                    var coord = Math.abs(self.coordsCols[i] - x);
                    if(coord < calc){
                        xx = self.coordsCols[i];
                        calc = coord;
                    }
                }

                return parseFloat((xx).toFixed(3));
            },

            __changeChipCol: function(e, self, cb){
                var xx = self.__calculateCol(e.x, self);
                if(xx != self.actualChip.x) {
                    //self.blockTurn = true;
                    self.actualChip.moveTo(
                        xx,
                        self.actualChip.y,
                        200,
                        0,
                        new CAAT.Interpolator().createLinearInterpolator(false, false),
                        function(e){
                            if (cb && typeof(cb) === "function") cb(self);
                        });
                }
            },

            __createChip: function(e, self){

                var xx = self.__calculateCol(e.x, self);

                self.actualChip = new FIAW.Actor.PlayerChip(self.turnGame)
                    .setBackgroundImage(this.turnChip[this.turnGame])
                    //.setSize(this.tileSize, this.tileSize)
                    //.setImageTransformation(CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE)
                    .rescale()
                    .setLocation(xx, this.tileSize/2);

                self.gameBoard.addChild(self.actualChip);

            },

            __dropChip: function(e, self, noMulti){
                var xx = self.__calculateCol(e.x, self);

                var cb = function(self){
                    var col = self.coordsCols.indexOf(parseFloat((self.actualChip.x).toFixed(3)));
                    var row = self.boardColsMap[col].lastIndexOf(false);

                    if(row != -1){

                        if(self.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE && !noMulti){
                            self.gamePlayManager.multiplayerManager.players[self.turnGame].match.sendDataToAllPlayers(JSON.stringify(["dropOn", col]));
                        }
                        self.stopTime();
                        self.pauseRemainingTime = null;
                        self.actualChip.setAlpha(1);

                        var yy = (self.gameBoard.height - self.tileSizeH/2) - (((self.boardColsMap[col].length-1) - row) * self.tileSizeH);

                        self.blockTurn = true;
                        self.removeGuide();
                        self.actualChip.moveTo(
                            self.actualChip.x,
                            yy,
                            1000,
                            100,
                            new CAAT.Interpolator().createBounceOutInterpolator(false),
                            function(e,t,obj){
                                obj.makeShadow(self);
                            });
                        self.boardColsMap[col][row] = self.actualChip;
                        self.blockTurn = false;
                        self.checkMap(col, row);
                        self.actualChip = null;

                        self.director.audioPlay('snd_drop_chip');

                    }else{
                        self.actualChip.setDiscardable(true).setExpired(true);
                        self.blockTurn = false;
                        self.actualChip = null;
                    }

                };

                if(xx !== self.actualChip.x){
                    self.__changeChipCol(e, self, cb);
                }else{
                    cb(self);
                }

            },

            showGuide: function(coord){
                var col = this.__calculateCol(coord, this);
                var colorTurn = (this.turnGame === FIAW.Globals.G_PLAYER_1) ? 'amarilla' : 'roja';

                var img1 = this.director.getImage('caida' + colorTurn + '1'),
                    img2 = this.director.getImage('caida' + colorTurn + '2');

                if(col != this.showGuideCoord){
                    this.showGuideCoord = col;

                    this.removeGuide();

                    var rows = this.boardColsMap[this.coordsCols.indexOf(col)].lastIndexOf(false);

                    if(rows !== -1){
                        var height = (rows+1) * this.tileSizeH;
                        this.showGuideContainer = new CAAT.Foundation.ActorContainer()
                            .setSize(this.tileSize, height)
                            .setGlobalAnchor(0.5, 0)
                            //.setFillStyle('black')
                            .setAlpha(0.6)
                            .setLocation(this.showGuideCoord, 0);

                        for(var i = 0; i <= rows; i++){
                            var hh = this.tileSizeH/2 + this.tileSizeH*i;
                            var alpha = 1/(rows+1);

                            var figure = (i == rows) ? img1 : img2;

                            var guideActor = new CAAT.Foundation.Actor()
                                .setFillStyle('#fff')
                                .setBackgroundImage(figure)
                                .setGlobalAnchor(0.5, 0.5)
                                .rescale()
                                .enableEvents(false)
                                .setAlpha(alpha*(i+1))
                                .setLocation(this.showGuideContainer.width/2, hh);

                            this.showGuideContainer.addChild(guideActor);
                        }

                        this.gameBoardSpace.addChild(this.showGuideContainer);
                    }
                }


            },

            removeGuide: function(){
                if(this.showGuideContainer) this.showGuideContainer.emptyChildren().setDiscardable(true).setExpired(true);
            },

            checkMap: function(col, row){

              var board = this.gamePlayManager.getBoardState();

              if(this.gamePlayManager.checkWinGame(col, row, this.turnGame, board)){
                this.__winGame();
              }else if(this.gamePlayManager.checkTieGame(board)){
                this.__tieGame();
              }else{
                this.__changeTurn();
              }

            },

            __winGame: function(arr){
                this.stopTime();
                this.countWin[this.turnGame]++;
                this.textWin[this.turnGame].setText(this.getCountWinString(this.turnGame));
                this.pauseButton.enableEvents(false);
                this.gameBoard.enableEvents(false);

                if(this.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE) this.gamePlayManager.multiplayerManager.incrementAndSaveScore(this.turnGame);

                this.gameOverWindow = new FIAW.Actor.GameOverWindow(this, 0);
                this.addChild(this.gameOverWindow);
            },

            __tieGame: function(){
                this.stopTime();
                this.pauseButton.enableEvents(false);
                this.gameBoard.enableEvents(false);

                this.gameOverWindow = new FIAW.Actor.GameOverWindow(this, 1);
                this.addChild(this.gameOverWindow);
            },

            __createTransBackground : function(){
                if(this.transBackground){
                    this.transBackground.setDiscardable(true).setExpired(true);
                }

                this.transBackground = new CAAT.Foundation.ActorContainer()
                    .setFillStyle('#000')
                    .setSize(this.director.width, this.director.height)
                    .setAlpha(0)
                    .enableEvents(false)
                    .setLocation(0,0);

                this.addChild(this.transBackground);

            },

            //Cambiamos el turno, si existe el parametro turn asignamos el indicado, en caso contrario asignamos el siguiente turno
            __changeTurn: function(){

                function cb(self){
                    self.turnGame = (self.turnGame === FIAW.Globals.G_PLAYER_1) ? FIAW.Globals.G_PLAYER_2 : FIAW.Globals.G_PLAYER_1;
                    self.pauseRemainingTime = null;
                    self.startTime();
                    self.showTurn();
                }

                var self = this;

                if(this.pause){

                    var checkPause = setInterval(function(){
                        if(!self.pause){
                            cb(self);
                            clearInterval(checkPause);
                        }
                    },1000/30);

                }else{
                    cb(self);
                }

            }

        };


    }

});