
CAAT.Module( {
    defines : "FIAW.Actor.MenuScene",
    depends : [
        "CAAT.Foundation.Scene",
        "FIAW.Actor.LoadingWindow"
    ],
    extendsClass : "CAAT.Foundation.Scene",

    extendsWith : function() {

        return {

            director: null,
            firstTime: true,

            backgroundContainer: null,
            gameContainer: null,
            gamePlayManager: null,

            gameScale: null,

            settingsPanel: null,
            settingsButton: null,
            settingOptions: [],
            settingMusicState: null,
            settingSoundState: null,

            removeAdsText: null,

            bgRanking : null,
            bgOnline : null,
            bgPlayerVsPlayer : null,
            bgPlayerVsMachine : null,

            transBackground : null,
            loadingWindow : null,

			__init: function(director){
				this.__super();
                this.director = director;

                this.setFillStyle('#000');
                this.initGamePlayManager();
                this.initBackgroundContainer();
                this.initGameContainer();
                this.initMainMenu();

				return this;
			},

            /* Al activar la escena por primera vez mostramos efectos de transición en sus actores */
            activated: function(){
                this.animateSceneIn();
                if(this.firstTime){
                    this.firstTime = false;

                    //Mostramos el banner de publicidad si procede.
                    if(navigator.isCocoonJS && FIAW.Utils.isAndroidOrIos()){
                        FIAW.Utils.adsManager.showBanner();
                    }

                    this.loadingWindow = new FIAW.Actor.LoadingWindow(this);
                    this.addChild(this.loadingWindow);

                }else{
                    if(navigator.isCocoonJS){
                        FIAW.Utils.rate.tryRate();
                    }
                }

            },

            initGamePlayManager: function(){
                this.gamePlayManager = new FIAW.Manager.GamePlayManager(this.director, this);
            },

            /* Se incia el fondo principal, adaptandolo el alto del contenedor
                al alto del navegador y ajustando el ancho proporcionalmente, además se centra
                en la pantalla. */
            initBackgroundContainer: function(){

                this.backgroundContainer = new CAAT.Foundation.ActorContainer()
                    .setBounds(0, 0, this.director.width, this.director.height);

                this.addChild(this.backgroundContainer);

                this.transBackground= new CAAT.Actor()
                    .setBackgroundImage( this.director.getImage('fondo'), true)
                    .enableEvents(false)
                    .setSize(this.director.width, this.director.height)
                    .setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE)
                    .setLocation(this.director.width/2,this.director.height/2)
                    .setGlobalAnchor(0.5,0.5)
                    .setAlpha(0)
                    .fixedPosition();

                this.menuBackgroundImage= new CAAT.Actor()
                    .setBackgroundImage( this.director.getImage('fondo-menu'), true)
                    .enableEvents(false)
                    .setSize(this.director.width, this.director.height)
                    .setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE)
                    .setLocation(this.director.width/2,this.director.height/2)
                    .setGlobalAnchor(0.5,0.5)
                    .fixedPosition();

                this.backgroundContainer.addChild(this.transBackground);
                this.backgroundContainer.addChild(this.menuBackgroundImage);

            },

            /* Contenedor principal y adaptación del background */
            initGameContainer: function(){

                this.gameScale = Math.min(this.director.width/this.director.getValueForKey("W"), this.director.height/this.director.getValueForKey("H"));

                this.gameContainer = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0.5)
                    .setScale(this.gameScale, this.gameScale)
                    //.setFillStyle('#9f0')
                    //.setAlpha(0.3)
                    .setBounds(
                        this.director.width/2,
                        0,
                        this.director.getValueForKey("W"),
                        this.director.getValueForKey("H")
                    )
                    .setLocation(this.director.width/2, this.director.height/2);

                this.addChild(this.gameContainer);
                if(!navigator.isCocoonJS){
                    this.menuBackgroundImage.setSize(this.gameContainer.width*this.gameScale,this.menuBackgroundImage.height);
                    this.transBackground.setSize(this.gameContainer.width*this.gameScale,this.menuBackgroundImage.height);
                }

            },

            __rankingButtonPressed: function(e, self){
                if(navigator.onLine){
                    self.gamePlayManager.multiplayerManager.showRanking();
                }else{
                    alert(FIAW.localeText.getString("T_DISCONNECTED", "Disconnected"));
                }
            },

            __playerVsPlayerPressed: function(e, self){
                self.gamePlayManager.activeGamePlay(FIAW.Globals.G_PLAYER_VS_PLAYER);
            },

            __playerVsOnlinePressed: function(e, self){
                self.gamePlayManager.multiplayerManager.startMultiplayerGame();
            },

            __playerVsComputerPressed: function(e, self){
                self.gamePlayManager.activeGamePlay(FIAW.Globals.G_PLAYER_VS_COMPUTER);
            },

            __shopButtonPressed: function(e, self){
                FIAW.Utils.inAppsManager.removeAds();
            },

            __ludeiButtonPressed: function(e){
                if(navigator.onLine) {
                    CocoonJS.App.openURL("http://redirect.ludei.com/2014");
                }else{
                    var textDisconnected = FIAW.localeText.getString("T_DISCONNECTED", "Disconnected");
                    console.log(textDisconnected);
                }
            },

            __moreGamesButtonPressed: function(e, self){
                if(!navigator.isCocoonJS){
                    if(navigator.onLine){
                        CocoonJS.App.openURL("https://www.ludei.com/marketplace/");
                    }else{
                        var textDisconnected = FIAW.localeText.getString("T_DISCONNECTED", "Disconnected");
                        alert(textDisconnected);
                    }
                }else{

                    var osType = false;
                    if(/ios/.test(navigator.userAgent)) {
                        osType = "moregames-ios";
                    }else if(/android/.test(navigator.userAgent)) {
                        osType = "moregames-android";
                    }

                    if(FIAW.Utils.isAmazonApp()) osType = "moregames-amazon";

                    if(osType){
                       var url = "webviews/moregames/" + osType + ".html";
                       self.loadingWindow.showWindow(FIAW.localeText.getString("T_LOADING", "LOADING"));
                       self.openMoreGamesWebView(url);
                    }else{
                        console.log('Unknow OS');
                    }

                }
            },

            __soundButtonPressed: function(e, self){
                self.gamePlayManager.soundButtonPressed(e, self);
            },

            __musicButtonPressed: function(e, self){
                self.gamePlayManager.musicButtonPressed(e, self);
            },

            openMoreGamesWebView: function(url){
                CocoonJS.App.onLoadInTheWebViewSucceed.addEventListenerOnce(function(pageUrl){
                    CocoonJS.App.disableTouchInCocoonJS();
                    CocoonJS.App.showTheWebView();
                });

                CocoonJS.App.onLoadInTheWebViewFailed.addEventListenerOnce(function(pageUrl){
                    var textCantShowMoreGames = FIAW.localeText.getString("T_CANT_SHOW_MORE_GAMES", "Can't show more games...");
                    alert(textCantShowMoreGames);
                });

                CocoonJS.App.loadInTheWebView(url);
            },

            initMainMenu: function(){
                
                this.initMenuStructure();

            },

            initMenuStructure: function(){
                var self = this;
                this.bgRanking = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('rankings'))
                    .setGlobalAnchor(0.5, 1)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(this.gameContainer.width/2, this.gameContainer.height)
                    .fixedPosition();

                this.bgOnline = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('online'))
                    .setGlobalAnchor(0.5, 1)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(this.gameContainer.width/2, this.bgRanking.y - 320)
                    .fixedPosition();

                this.bgPlayerVsMachine = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('playervsmachine'))
                    .setGlobalAnchor(0.5, 1)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(this.gameContainer.width/2, this.bgOnline.y - 320)
                    .fixedPosition();

                this.bgPlayerVsPlayer = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('playervsplayer'))
                    .setGlobalAnchor(0.5, 1)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(this.gameContainer.width/2, this.bgPlayerVsMachine.y - 320)
                    .fixedPosition();

                this.settingsPanel = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('setings-desplegado'))
                    .setGlobalAnchor(0.5, 1)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(this.gameContainer.width/2, this.bgPlayerVsPlayer.y - 320)
                    .fixedPosition();

                var settingIcon = new CAAT.Foundation.Actor()
                    .setBackgroundImage(this.director.getImage('boton-setings'))
                    .setGlobalAnchor(0.5, 0.5)
                    .rescale()
                    .setAlpha(0)
                    .enableEvents(false);

                this.settingsButton = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0.5)
                    .enableEvents(true)
                    .setSize(settingIcon.width * 2, settingIcon.height*2)
                    .setScale(this.gameScale, this.gameScale)
                    .setLocation(this.gameContainer.x + (this.gameContainer.width/2.5 * this.gameScale), settingIcon.height*this.gameScale);

                settingIcon.setLocation(this.settingsButton.width/2, this.settingsButton.height/2);
                this.settingsButton.addChild(settingIcon);

                this.settingsButton.pulsed = false;

                this.settingsButton.mouseDown = function(e){
                    self.__settingButtonPressed(e, self);
                };

                this.gameContainer.addChild(this.bgRanking);
                this.gameContainer.addChild(this.bgOnline);
                this.gameContainer.addChild(this.bgPlayerVsMachine);
                this.gameContainer.addChild(this.bgPlayerVsPlayer);

                var playerVsPlayer = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0.5)
                    .setSize(this.gameContainer.width, 280)
                    //.setFillStyle('#fff')
                    .setAsButton(null, 0,0,0,0,function(e){
                        self.director.audioPlay('snd_menu_button');
                        self.__playerVsPlayerPressed(e, self);
                    });

                var textPlayerVsPlayer = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-white"))
                    .setText(FIAW.localeText.getString("T_PLAYER_VS_PLAYER", "PLAYER VS PLAYER"))
                    .setTextAlign("center")
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false);

                playerVsPlayer.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);
                textPlayerVsPlayer.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);

                var playerVsMachine = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0.5, 0.5)
                    .setSize(this.gameContainer.width, 280)
                    //.setFillStyle('#f00')
                    .setAsButton(null, 0,0,0,0,function(e){
                        self.director.audioPlay('snd_menu_button');
                        self.__playerVsComputerPressed(e, self);
                    });

                var textPlayerVsMachine = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-white"))
                    .setText(FIAW.localeText.getString("T_PLAYER_VS_COMPUTER", "PLAYER VS COMPUTER"))
                    .setTextAlign("center")
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false);

                playerVsMachine.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);
                textPlayerVsMachine.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);

                //Botones multijugador y ranking solo el cocoonjs, posiciones de playerVsPlayer y vs Machine cambiadas
                if(navigator.isCocoonJS && FIAW.Utils.isAndroidOrIos() && !FIAW.Utils.isAmazonApp()){

                    playerVsPlayer.setLocation(this.bgPlayerVsPlayer.width/2, this.bgPlayerVsPlayer.y - 470);
                    textPlayerVsPlayer.setLocation(this.bgPlayerVsPlayer.width/2, this.bgPlayerVsPlayer.y + 120);
                    this.bgPlayerVsPlayer.addChild(textPlayerVsPlayer);

                    playerVsMachine.setLocation(this.bgPlayerVsMachine.width/2, this.bgPlayerVsMachine.y - 470);
                    textPlayerVsMachine.setLocation(this.bgPlayerVsMachine.width/2, this.bgPlayerVsMachine.y + 120);
                    this.bgPlayerVsMachine.addChild(textPlayerVsMachine);

                    var playerVsOnline = new CAAT.Foundation.ActorContainer()
                        .setGlobalAnchor(0.5, 0.5)
                        .setSize(this.gameContainer.width, 280)
                        //.setFillStyle('#0f0')
                        .setAsButton(null, 0,0,0,0,function(e){
                            self.director.audioPlay('snd_menu_button');
                            self.__playerVsOnlinePressed(e, self);
                        });

                    var textPlayerVsOnline = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap-white"))
                        .setText(FIAW.localeText.getString("T_ONLINE_MULTIPLAYER", "ONLINE MULTIPLAYER"))
                        .setTextAlign("center")
                        .setBaseline("middle")
                        .cacheAsBitmap()
                        .enableEvents(false);

                    playerVsOnline.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);
                    textPlayerVsOnline.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);

                    playerVsOnline.setLocation(this.bgOnline.width/2, this.bgOnline.y - 470);
                    textPlayerVsOnline.setLocation(this.bgOnline.width/2, this.bgOnline.y + 120);
                    this.bgOnline.addChild(textPlayerVsOnline);

                    this.gameContainer.addChild(playerVsOnline);

                    var rankingButton = new CAAT.Foundation.ActorContainer()
                        .setGlobalAnchor(0.5, 0.5)
                        .setSize(this.gameContainer.width, 280)
                        //.setFillStyle('#00f')
                        .setAsButton(null, 0,0,0,0,function(e){
                            self.director.audioPlay('snd_menu_button');
                            self.__rankingButtonPressed(e, self);
                        });

                    var textRanking = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap-white"))
                        .setText(FIAW.localeText.getString("T_RANKING", "RANKING"))
                        .setTextAlign("center")
                        .setBaseline("middle")
                        .cacheAsBitmap()
                        .enableEvents(false);

                    rankingButton.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);
                    textRanking.setRotationAnchored(-18 * Math.PI / 180, 0.5, 0.5);

                    rankingButton.setLocation(this.bgRanking.width/2, this.bgRanking.y - 470);
                    textRanking.setLocation(this.bgRanking.width/2, this.bgRanking.y + 120);
                    this.bgRanking.addChild(textRanking);

                    this.gameContainer.addChild(rankingButton);

                }else{
                    playerVsPlayer.setLocation(this.bgPlayerVsMachine.width/2, this.bgPlayerVsMachine.y - 470);
                    textPlayerVsPlayer.setLocation(this.bgPlayerVsMachine.width/2, this.bgPlayerVsMachine.y + 120);
                    this.bgPlayerVsMachine.addChild(textPlayerVsPlayer);

                    playerVsMachine.setLocation(this.bgOnline.width/2, this.bgOnline.y - 470);
                    textPlayerVsMachine.setLocation(this.bgOnline.width/2, this.bgOnline.y + 120);
                    this.bgOnline.addChild(textPlayerVsMachine);
                }

                this.gameContainer.addChild(playerVsPlayer);
                this.gameContainer.addChild(playerVsMachine);
                this.gameContainer.addChild(this.settingsPanel);
                this.addChild(this.settingsButton);

                this.initSettingOptions();

            },

            initSettingOptions: function(){
                var self = this;
                var soundButton = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0, 0)
                    .enableEvents(true)
                    .setVisible(false)
                    .setLocation(0, 10);

                var soundText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-white"))
                    .setText(FIAW.localeText.getString("T_SOUND","SOUND"))
                    .setTextAlign("left")
                    .setBaseline("top")
                    .setGlobalAnchor(0, 0)
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(0, 0);

                soundButton.addChild(soundText);

                this.settingSoundState = new CAAT.Foundation.Actor()
                    .setBackgroundImage(
                        this.director.getImage( (FIAW.soundEnabled) ? 'ok' : 'off' )
                    )
                    .setGlobalAnchor(0, 0.5)
                    .enableEvents(false)
                    .rescale()
                    .setLocation(soundText.width + 50, soundText.height/2 + 35);

                soundButton.setSize(soundText.width + this.settingSoundState.width + 50, soundText.height -20);

                soundButton.addChild(this.settingSoundState);

                soundButton.mouseDown = function(e){
                    self.director.audioPlay('snd_menu_button');
                    self.__soundButtonPressed(e, self);
                };

                this.gameContainer.addChild(soundButton);
                this.settingOptions.push(soundButton);

                var musicButton = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0, 0)
                    .enableEvents(true)
                    .setVisible(false)
                    .setLocation(0, 10 + ((soundText.height) * this.settingOptions.length));

                var musicText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-white"))
                    .setText(FIAW.localeText.getString("T_MUSIC","MUSIC"))
                    .setTextAlign("left")
                    .setBaseline("top")
                    .setGlobalAnchor(0, 0)
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(0, 0);

                musicButton.addChild(musicText);

                this.settingMusicState = new CAAT.Foundation.Actor()
                    .setBackgroundImage(
                        this.director.getImage( (FIAW.musicEnabled) ? 'ok' : 'off' )
                    )
                    .setGlobalAnchor(0, 0.5)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(musicText.width + 50, musicText.height/2 + 35);

                musicButton.setSize(musicText.width + this.settingMusicState.width + 50, musicText.height - 20);

                musicButton.addChild(this.settingMusicState);

                musicButton.mouseDown = function(e){
                    self.director.audioPlay('snd_menu_button');
                    self.__musicButtonPressed(e, self);
                };

                this.gameContainer.addChild(musicButton);
                this.settingOptions.push(musicButton);

                if(navigator.isCocoonJS && FIAW.Utils.isAndroidOrIos()){
                    if(FIAW.Utils.rate.menuCanShowRate()){
                        var rateMenu = new CAAT.Foundation.UI.TextActor()
                            .setFont(FIAW.Utils.getFont("sixcap-white"))
                            .setText(FIAW.localeText.getString("T_RATE_IT","RATE IT") + " ")
                            .setTextAlign("left")
                            .setBaseline("top")
                            .setGlobalAnchor(0, 0)
                            .cacheAsBitmap()
                            .enableEvents(true)
                            .setVisible(false)
                            .setLocation(0, 10 + ((soundText.height) * this.settingOptions.length));

                        rateMenu.mouseDown = function(e){
                            self.director.audioPlay('snd_menu_button');
                            FIAW.Utils.rate.rateMenu();
                            rateMenu.setDiscardable(true).setExpired(true);
                        };

                        this.gameContainer.addChild(rateMenu);
                        this.settingOptions.push(rateMenu);
                    }

                    if(FIAW.adsEnabled){
                        this.removeAdsText = new CAAT.Foundation.UI.TextActor()
                            .setFont(FIAW.Utils.getFont("sixcap-red"))
                            .setText(FIAW.localeText.getString("T_REMOVE_ADS","REMOVE ADS") + " ")
                            .setTextAlign("left")
                            .setBaseline("top")
                            .setGlobalAnchor(0, 0)
                            .cacheAsBitmap()
                            .enableEvents(true)
                            .setVisible(false)
                            .setLocation(0, 10 + ((soundText.height) * this.settingOptions.length));

                        this.removeAdsText.mouseDown = function(e){
                            self.director.audioPlay('snd_menu_button');
                            self.__shopButtonPressed(e, self);
                        };

                        this.gameContainer.addChild(this.removeAdsText);
                        this.settingOptions.push(this.removeAdsText);
                    }

                    var restorePurchases = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(FIAW.localeText.getString("T_RESTORE_PURCHASES","RESTORE PURCHASES") + " ")
                        .setTextAlign("left")
                        .setBaseline("top")
                        .setGlobalAnchor(0, 0)
                        .cacheAsBitmap()
                        .enableEvents(true)
                        .setVisible(false)
                        .setLocation(0, 10 + ((soundText.height) * this.settingOptions.length));

                    restorePurchases.mouseDown = function(e){
                        self.director.audioPlay('snd_menu_button');
                        self.__restorePurchasesPressed(e, self);
                    };

                    this.gameContainer.addChild(restorePurchases);
                    this.settingOptions.push(restorePurchases);
                }

                var moreGamesText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-green"))
                    .setText(FIAW.localeText.getString("T_MORE_GAMES","MORE GAMES") + " ")
                    .setTextAlign("left")
                    .setBaseline("top")
                    .setGlobalAnchor(0, 0)
                    .cacheAsBitmap()
                    .enableEvents(true)
                    .setVisible(false)
                    .setLocation(0, 10 + ((soundText.height) * this.settingOptions.length));

                moreGamesText.mouseDown = function(e){
                    self.director.audioPlay('snd_menu_button');
                    self.__moreGamesButtonPressed(e, self);
                };

                this.gameContainer.addChild(moreGamesText);
                this.settingOptions.push(moreGamesText);

                var moreInfoText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-green"))
                    .setText(FIAW.localeText.getString("T_MORE_INFO","MORE INFO") + " ")
                    .setTextAlign("left")
                    .setBaseline("top")
                    .setGlobalAnchor(0, 0)
                    .cacheAsBitmap()
                    .enableEvents(true)
                    .setVisible(false)
                    .setLocation(0, 10 + ((soundText.height) * this.settingOptions.length));

                moreInfoText.mouseDown = function(e){
                    self.director.audioPlay('snd_menu_button');
                    self.__ludeiButtonPressed(e, self);
                };

                this.gameContainer.addChild(moreInfoText);
                this.settingOptions.push(moreInfoText);


            },

            __restorePurchasesPressed : function(e, self){
                FIAW.Utils.inAppsManager.restorePurchases();
            },

            __settingButtonPressed: function(e, self){
                var time = 300;

                if(!self.settingsButton.pulsed){
                    self.settingsButton.pulsed = true;
                    self.settingsPanel.enableEvents(true);
                    self.__showSettingsPanel(time);
                }else{
                    self.settingsButton.pulsed = false;
                    self.settingsPanel.enableEvents(false);
                    self.__hideSettingsPanel(time);
                }
            },

            __showSettingsPanel: function(duration){
                var behavior = new CAAT.Behavior.RotateBehavior()
                    .setValues(1.5*Math.PI, 0)
                    .setDelayTime(0, duration)
                    .setCycle(false);

                this.settingsButton.addBehavior(behavior).enableEvents(false);

                var self = this;
                this.settingsPanel.moveTo(
                    self.settingsPanel.x,
                    self.bgRanking.y,
                    duration,
                    0,
                    new CAAT.Interpolator().createLinearInterpolator(false, false),
                    function(e){
                            
                        self.settingsButton.enableEvents(true);
                });


                var cb = function(e,t,obj){obj.enableEvents(true);};

                setTimeout(function(){
                    for(var i = 0; i < self.settingOptions.length; i++){

                        //Si es cocoonjs emitimos la animación desde fuera de la vista.
                        if(navigator.isCocoonJS) self.settingOptions[i].x = -self.settingOptions[i].width;

                        self.settingOptions[i].setVisible(true);
                        self.settingOptions[i].moveTo(
                            self.gameContainer.width/6,
                            self.settingOptions[i].y,
                            100 + (i*90),
                            0,
                            new CAAT.Interpolator().createLinearInterpolator(false, false),
                            cb
                        );
                    }
                }, 100);

            },

            __hideSettingsPanel: function(duration){
                var behavior = new CAAT.Behavior.RotateBehavior()
                    .setValues(0, 1.5*Math.PI)
                    .setDelayTime(0, duration+150)
                    .setCycle(false);

                this.settingsButton.addBehavior(behavior).enableEvents(false);

                var self = this;
                this.settingsPanel.moveTo(
                    self.settingsPanel.x,
                    self.settingsPanel.fixedY,
                    duration,
                    0,
                    new CAAT.Interpolator().createLinearInterpolator(false, false),
                    function(e){
                            
                        self.settingsButton.enableEvents(true);
                });

                var hideOption = function(e, t, obj){
                    obj.setVisible(false).enableEvents(false);
                };

                for(var i = 0; i < this.settingOptions.length; i++){
                    var n = (this.settingOptions.length-1)-i;
                    this.settingOptions[n].moveTo(
                        0,
                        this.settingOptions[n].y,
                        70 + (i*50),
                        0,
                        new CAAT.Interpolator().createLinearInterpolator(false, false),
                        hideOption
                    );
                }
            },

            //Se hace un FadeIn de la escena
            animateSceneIn: function(){
                this.enableEvents(false);
                this.settingsButton.getChildAt(0).setAlpha(0);
                var self = this;
                var offSetY = this.menuBackgroundImage.height - (this.gameContainer.height * this.gameContainer.scaleY);

                var cbMouseOff = function(e){
                    if(self.mouseEnabled){
                        self.enableEvents(false);
                    }
                };

                this.menuBackgroundImage.setLocation(this.menuBackgroundImage.x, this.menuBackgroundImage.height*2);
                this.menuBackgroundImage.moveTo(
                    this.menuBackgroundImage.x,
                    this.menuBackgroundImage.fixedY,
                    1000,
                    0,
                    new CAAT.Interpolator().createLinearInterpolator(false, false),
                    cbMouseOff
                );

                this.bgRanking.setLocation(this.bgRanking.fixedX, -offSetY);
                this.bgRanking.moveTo(
                    this.bgRanking.fixedX,
                    this.bgRanking.fixedY,
                    1000,
                    500,
                    new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                    cbMouseOff
                );

                this.bgOnline.setLocation(this.bgOnline.fixedX, -offSetY);
                this.bgOnline.moveTo(
                    this.bgOnline.fixedX,
                    this.bgOnline.fixedY,
                    1000,
                    700,
                    new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                    cbMouseOff
                );

                this.bgPlayerVsMachine.setLocation(this.bgPlayerVsMachine.fixedX, -offSetY);
                this.bgPlayerVsMachine.moveTo(
                    this.bgPlayerVsMachine.fixedX,
                    this.bgPlayerVsMachine.fixedY,
                    1000,
                    900,
                    new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                    cbMouseOff
                );

                this.bgPlayerVsPlayer.setLocation(this.bgPlayerVsPlayer.fixedX, -offSetY);
                this.bgPlayerVsPlayer.moveTo(
                    this.bgPlayerVsPlayer.fixedX,
                    this.bgPlayerVsPlayer.fixedY,
                    1000,
                    1100,
                    new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                    cbMouseOff
                );

                this.settingsPanel.setLocation(this.settingsPanel.fixedX, -offSetY);
                this.settingsPanel.moveTo(
                    this.settingsPanel.fixedX,
                    this.settingsPanel.fixedY,
                    1000,
                    1300,
                    new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                    function(e){
                        self.enableEvents(true);
                        self.settingsButton.getChildAt(0).setAlpha(1);
                    }
                );

            },

            animateSceneOut: function(gameMode, cb){
                cb = cb || function(e){};

                this.enableEvents(false);
                this.settingsButton.getChildAt(0).setAlpha(0);
                var self = this;
                var offSetY = this.menuBackgroundImage.height - (this.gameContainer.height * this.gameContainer.scaleY);

                this.transBackground.setAlpha(1);

                this.menuBackgroundImage.moveTo(
                    this.menuBackgroundImage.x,
                    this.menuBackgroundImage.height*2,
                    1200,
                    200,
                    new CAAT.Interpolator().createLinearInterpolator(false, false),
                    function(e){}
                );

                var layers = [
                    this.settingsPanel,
                    this.bgPlayerVsPlayer,
                    this.bgPlayerVsMachine,
                    this.bgOnline,
                    this.bgRanking
                ];

                var layerWait;

                if(navigator.isCocoonJS && FIAW.Utils.isAndroidOrIos()){
                    layerWait = gameMode == FIAW.Globals.G_PLAYER_VS_PLAYER ? this.bgPlayerVsPlayer : this.bgPlayerVsMachine;
                }else{
                    layerWait = gameMode == FIAW.Globals.G_PLAYER_VS_PLAYER ? this.bgPlayerVsMachine : this.bgOnline;
                }

                var setCallback = function(e){};

                for(var i = 0; i < layers.length; i++){

                    callback = setCallback;

                    var timeStart = 200*i;
                    if(layers[i] === layerWait){
                        timeStart = 200*layers.length + 300;
                        callback = cb;
                    }
                    layers[i].moveTo(
                        layers[i].x,
                        -offSetY,
                        500,
                        timeStart,
                        new CAAT.Interpolator().createExponentialInOutInterpolator(3,false),
                        callback
                    );
                }

            },

        };
    }

});