CAAT.Module( {
    defines : "FIAW.Actor.PauseWindow",
    depends : [
        "CAAT.Foundation.ActorContainer"
    ],
    extendsClass : "CAAT.Foundation.ActorContainer",

    extendsWith : function() {

        return {

            gamePlayScene: null,
            gamePlayManager: null,
            director: null,
            menuOpened: false,
            optionsMenu: null,
            moveHeight: null,

            playAgainButton: null,
            gotoMenuButton: null,
            removeAdsButton: null,
            removeAdsText: null,
            musicButton: null,
            soundButton: null,

            musicStatusText: null,
            soundStatusText: null,

            __init: function(gamePlayScene){
                this.__super();

                this.gamePlayScene = gamePlayScene;
                this.director = this.gamePlayScene.director;
                this.gamePlayManager = this.gamePlayScene.gamePlayManager;

                this.initContainer();
                return this;
            },

            initContainer: function(){

                var self = this;

                this.setBackgroundImage(this.director.getImage('corte'))
                    .rescale()
                    .setScale(this.gamePlayScene.gameScale, this.gamePlayScene.gameScale)
                    .setGlobalAnchor(0.5, 0)
                    .enableEvents(false)
                    .setVisible(false)
                    .setLocation(this.gamePlayScene.gameContainer.width/2, this.gamePlayScene.topHud.height);

                this.setSize(this.gamePlayScene.gameBackgroundImage.width/this.gamePlayScene.gameScale, this.height);

                this.moveHeight = this.height;

                this.optionsMenu = new CAAT.Foundation.ActorContainer()
                    .setGlobalAnchor(0, 0)
                    .setSize(this.width, this.height)
                    .setLocation(0, 0);

                this.playAgainButton = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage("pause-menu-button"))
                    .rescale()
                    .setGlobalAnchor(1, 0)
                    .setLocation(this.optionsMenu.width, 5);

                var playAgainText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-red"))
                    .setText(FIAW.localeText.getString("T_PLAY_AGAIN", "PLAY AGAIN") + " ")
                    .setTextAlign("left")
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(30, this.playAgainButton.height/2);

                this.playAgainButton.addChild(playAgainText);
                this.optionsMenu.addChild(this.playAgainButton);

                this.gotoMenuButton = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage("pause-menu-button"))
                    .rescale()
                    .setGlobalAnchor(1, 0)
                    .setLocation(this.optionsMenu.width, 6 + this.playAgainButton.height);

                var gotoMenuText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-red"))
                    .setText(FIAW.localeText.getString("T_GOTO_MENU", "GO TO MENU") + " ")
                    .setTextAlign("left")
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(30, this.gotoMenuButton.height/2);

                this.gotoMenuButton.addChild(gotoMenuText);
                this.optionsMenu.addChild(this.gotoMenuButton);

                this.removeAdsButton = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage("pause-menu-button"))
                    .rescale()
                    .setGlobalAnchor(1, 0)
                    .setLocation(this.optionsMenu.width, 7 + this.playAgainButton.height + this.gotoMenuButton.height);

                if(FIAW.adsEnabled){
                     this.removeAdsText = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap-red"))
                        .setText(FIAW.localeText.getString("T_REMOVE_ADS", "REMOVE ADS") + " ")
                        .setTextAlign("left")
                        .setBaseline("middle")
                        .cacheAsBitmap()
                        .enableEvents(false)
                        .setLocation(30, this.removeAdsButton.height/2);

                    this.removeAdsButton.addChild(this.removeAdsText);

                    this.removeAdsButton.mouseDown = function(e){
                        self.removeAdsPressed(e, self);
                    };
                }

                this.optionsMenu.addChild(this.removeAdsButton);

                this.gotoMenuButton.mouseDown = function(e){
                    self.gotoMenuPressed(e, self);
                };

                this.playAgainButton.mouseDown = function(e){
                    self.playAgainPressed(e, self);
                };

                var centeredContainer = new CAAT.Foundation.ActorContainer()
                    .setSize(this.width/3*2, this.height)
                    //.setFillStyle('green')
                    .setGlobalAnchor(0.5, 0.5)
                    .setLocation((this.width - this.removeAdsButton.width)/2, this.optionsMenu.height/2)
                    .setAlpha(0.8);

                this.optionsMenu.addChild(centeredContainer);

                this.musicButton = new CAAT.Foundation.ActorContainer()
                    .setSize(this.width/3, 180)
                    //.setFillStyle('blue')
                    .setGlobalAnchor(1, 0.5);

                var musicPolygon = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('poligono'))
                    .setGlobalAnchor(0, 0.5)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(0, this.musicButton.height/2);

                this.musicButton.addChild(musicPolygon);

                //Fix centered container
                centeredContainer.setSize(this.width/3*2 - musicPolygon.width/2, centeredContainer.height);
                this.musicButton.setLocation(centeredContainer.width, this.height/6 * 2);

                this.musicStatusText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont((FIAW.musicEnabled) ? "sixcap-red" : "sixcap-black"))
                    .setText(((FIAW.musicEnabled) ? FIAW.localeText.getString("T_ON", "ON") : FIAW.localeText.getString("T_OFF", "OFF")) + " ")
                    .setTextAlign("center")
                    .setScale(0.75, 0.75)
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(musicPolygon.width/2, musicPolygon.height/2 - 5);

                musicPolygon.addChild(this.musicStatusText);

                var musicText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-red"))
                    .setText(FIAW.localeText.getString("T_MUSIC", "MUSIC") + " ")
                    .setTextAlign("left")
                    //.setScale(0.75, 0.75)
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(musicPolygon.width + 5, this.musicButton.height/2 - 10);

                this.musicButton.addChild(musicText);

                centeredContainer.addChild(this.musicButton);

                this.soundButton = new CAAT.Foundation.ActorContainer()
                    .setSize(this.width/3, 180)
                    //.setFillStyle('blue')
                    .setGlobalAnchor(1, 0.5)
                    .setLocation(centeredContainer.width - this.width/3 + musicPolygon.width/2, this.height - (this.height/6 * 2));

                var soundPolygon = new CAAT.Foundation.ActorContainer()
                    .setBackgroundImage(this.director.getImage('poligono'))
                    .setGlobalAnchor(1, 0.5)
                    .rescale()
                    .enableEvents(false)
                    .setLocation(this.soundButton.width, this.soundButton.height/2);

                this.soundButton.addChild(soundPolygon);

                this.soundStatusText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont((FIAW.soundEnabled) ? "sixcap-red" : "sixcap-black"))
                    .setText(((FIAW.soundEnabled) ? FIAW.localeText.getString("T_ON", "ON") : FIAW.localeText.getString("T_OFF", "OFF")) + " ")
                    .setTextAlign("center")
                    .setScale(0.75, 0.75)
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(soundPolygon.width/2, soundPolygon.height/2 - 5);

                soundPolygon.addChild(this.soundStatusText);

                var soundText = new CAAT.Foundation.UI.TextActor()
                    .setFont(FIAW.Utils.getFont("sixcap-red"))
                    .setText(FIAW.localeText.getString("T_SOUND", "SOUND") + " ")
                    .setTextAlign("right")
                    //.setScale(0.75, 0.75)
                    .setBaseline("middle")
                    .cacheAsBitmap()
                    .enableEvents(false)
                    .setLocation(this.soundButton.width - soundPolygon.width - 2, this.soundButton.height/2 - 10);

                this.soundButton.addChild(soundText);

                centeredContainer.addChild(this.soundButton);

                this.soundButton.mouseDown = function(e){
                    self.soundPressed(e, self);
                };

                this.musicButton.mouseDown = function(e){
                    self.musicPressed(e, self);
                };

                this.addChild(this.optionsMenu);
            },

            playAgainPressed: function(e, self){
                self.__closeMenu(function(){
                    self.gamePlayScene.resetGame();
                    self.gamePlayScene.__initialize();
                });
                self.director.audioPlay('snd_menu_button');
            },

            removeAdsPressed: function(e, self){
                FIAW.Utils.inAppsManager.removeAds();
                self.director.audioPlay('snd_menu_button');
            },

            gotoMenuPressed: function(e, self){
                self.__closeMenu(function(){

                    self.gamePlayManager.returnMenuScene();
                });
                self.director.audioPlay('snd_menu_button');
            },

            musicPressed: function(e, self){
                self.gamePlayManager.musicButtonPressed(e, self);
                self.director.audioPlay('snd_menu_button');
            },

            soundPressed: function(e, self){
                self.gamePlayManager.soundButtonPressed(e, self);
                self.director.audioPlay('snd_menu_button');
            },

            menu: function(){
                if(this.menuOpened){
                    this.__closeMenu();
                }else{
                    this.__openMenu();
                }

                this.menuOpened = !this.menuOpened;
            },

            __openMenu: function(){
                var self = this;
                this.setVisible(true);
                this.gamePlayScene.pauseButton.enableEvents(false);
                var behavior = new CAAT.Behavior.ScaleBehavior()
                    .setValues(1, 1, 0, 1, 0, 0)
                    .setDelayTime(0, 500)
                    .setCycle(false);

                behavior.addListener({
                    behaviorExpired: function(behavior, time, actor){
                        actor.enableEvents(true);
                        self.gamePlayScene.pauseButton.enableEvents(true);
                    }
                });

                this.addBehavior(behavior);

                var cb = function(){};

                var gameBoard = [this.gamePlayScene.gameBoardSpace, this.gamePlayScene.gameBoard];
                for(var i = 0; i < gameBoard.length; i++){
                    gameBoard[i].moveTo(
                        gameBoard[i].x,
                        gameBoard[i].y + (this.moveHeight + 20) * this.gamePlayScene.gameScale,
                        500,
                        0,
                        new CAAT.Interpolator().createLinearInterpolator(false, false),
                        cb);
                }
            },

            __closeMenu: function(callback){
                var self = this;
                this.gamePlayScene.pauseButton.enableEvents(false);
                this.enableEvents(false);

                var behavior = new CAAT.Behavior.ScaleBehavior()
                    .setValues(1, 1, 1, 0, 0, 0)
                    .setDelayTime(0, 500)
                    .setCycle(false);

                behavior.addListener({
                    behaviorExpired: function(behavior, time, actor){
                        actor.setVisible(false);
                        self.gamePlayScene.pauseButton.enableEvents(true);
                        if(callback && typeof callback == 'function'){
                            callback();
                        }
                    }
                });

                this.addBehavior(behavior);

                var cb = function(){};

                var gameBoard = [this.gamePlayScene.gameBoardSpace, this.gamePlayScene.gameBoard];
                for(var i = 0; i < gameBoard.length; i++){
                    gameBoard[i].moveTo(
                        gameBoard[i].x,
                        gameBoard[i].y - (this.moveHeight + 20)* this.gamePlayScene.gameScale,
                        500,
                        0,
                        new CAAT.Interpolator().createLinearInterpolator(false, false),
                        cb);
                }
            }

        };

    }

});