(function(){
    var menuScene, loadTime;

    var W = 1535,
        H = 2048;

    /* Creo el objeto Utils en el namespace FIAW para que esté siempre accesible. */
    function __initUtils(){
        FIAW.Utils = new FIAW.Game.Utils();
    }

    /* Se crea el elemento canvas teniendo en cuenta el ancho/alto del navegador para hacer el reescalado*/
    function __createCanvas(){

        var c = document.createElement(
            (navigator.isCocoonJS) ? "screencanvas" : "canvas"
        );
        document.body.appendChild(c);

        var director = new CAAT.Director().initialize(window.innerWidth, window.innerHeight, c).setClear(true);

        director.easeNone = easeNone;

        director.setValueForKey("W", W);
        director.setValueForKey("H", H);
        director.setValueForKey("WORLD_SCREEN_SCALE", Math.min( director.width/W, director.height/H ));
        director.setValueForKey("IMS", ( window.innerHeight>2000 ) ? 1 : 0.5 );

        rescaleFunctions();
        CAAT.Foundation.Actor.prototype.fixedPosition = function(){
            this.fixedX = this.x;
            this.fixedY = this.y;
            return this;
        };
        CAAT.Foundation.ActorContainer.prototype.fixedPosition = CAAT.Foundation.Actor.prototype.fixedPosition;

        return director;
    }

    //Función auxiliar para cambiar entre escenas sin animación
    function easeNone(inSceneIndex, outSceneIndex, time) {

        if (inSceneIndex === this.getCurrentSceneIndex()) {
            return;
        }

        var ssin = this.scenes[ inSceneIndex ];
        var sout = this.scenes[ outSceneIndex ];

        if (!CAAT.__CSS__ && CAAT.CACHE_SCENE_ON_CHANGE) {
            this.renderToContext(this.transitionScene.ctx, sout);
            sout = this.transitionScene;
        }

        ssin.setExpired(false);
        sout.setExpired(false);

        ssin.mouseEnabled = false;
        sout.mouseEnabled = false;

        ssin.resetTransform();
        sout.resetTransform();

        ssin.setLocation(0, 0);
        sout.setLocation(0, 0);

        ssin.alpha = 1;
        sout.alpha = 1;

        var self = this;

        ssin.createTimer( ssin.time, time, function() {
            self.easeEnd(ssin, true);
            self.easeEnd(sout, false);
        });

        this.childrenList = [];

        sout.goOut(ssin);
        ssin.getIn(sout);

        this.addChild(ssin);
        this.addChild(sout);
    }

    /* Función del para el reescalado de assets */
    function rescaleFunctions(){

        CAAT.Foundation.Actor.prototype.rescale= function(fix) {
            fix = fix || false;

            var ims= CAAT.currentDirector.getValueForKey("IMS");
            this.width/=ims;
            this.height/=ims;
            if(fix) this.width += 2;
            this.setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE );
            return this;
        };
        CAAT.Foundation.ActorContainer.prototype.rescale= function(fix) {
            CAAT.Foundation.ActorContainer.superclass.rescale.call(this);
            for( var i=0; i<this.getNumChildren(); i++ ) {
                this.getChildAt(i).rescale(fix);
            }
            return this;
        };
        CAAT.Foundation.Actor.prototype.rescaleToScreen= function() {
            var ims= CAAT.currentDirector.getValueForKey("IMS");
            var wss= CAAT.currentDirector.getValueForKey("WORLD_SCREEN_SCALE");
            var w= this.width*wss/ims;
            var h= this.height*wss/ims;
            this.setSize(w,h);
            this.setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE );
            return this;
        };
        CAAT.Foundation.ActorContainer.prototype.rescaleToScreen= function() {
            CAAT.Foundation.ActorContainer.superclass.rescaleToScreen.call(this);
            for( var i=0; i<this.getNumChildren(); i++ ) {
                this.getChildAt(i).rescaleToScreen();
            }
            var ims= CAAT.currentDirector.getValueForKey("IMS");
            var wss= CAAT.currentDirector.getValueForKey("WORLD_SCREEN_SCALE");
            var w= this.width*wss/ims;
            var h= this.height*wss/ims;
            this.setSize(w,h);
            this.setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE );
            return this;
        };
    }

    /* Se cargan las imagenes de carga y se muestran mientras se cargan el resto de recursos */
    function __showSplashScreen(director){

        var splashScene =  director.createScene().setFillStyle('#000');

        var hd = director.getValueForKey("IMS")===1 ? "-hd" : "";

        new CAAT.ImagePreloader().loadImages(
            [
                { id: 'loadingImg', url: 'assets/img/loading'+hd+'.png'},
                { id: 'font', url: 'assets/img/fonts/sixcap.png'}
            ],

            function(counter, images){

                if(counter == images.length){

                    director.setImagesCache(images);

                    var rows = 3,
                        cols = 4;

                    var sprite = new CAAT.Foundation.SpriteImage().initialize(
                            director.getImage('loadingImg'),
                            rows,
                            cols
                        )
                        .addAnimation(
                            "idle",
                            [0,1,2,3,4,5,6,7,8,9,10,11],
                            30,
                            null
                        );

                    var loadingImg = new CAAT.Actor()
                        .setBackgroundImage( sprite, true )
                        .enableEvents(false)
                        .setImageTransformation( CAAT.Foundation.SpriteImage.TR_FIXED_TO_SIZE)
                        .setLocation( director.width/2, director.height/2 - sprite.height/rows)
                        .setGlobalAnchor(0.5, 0.5);

                    loadingImg.playAnimation("idle");

                    FIAW.Utils.createFont(director, "sixcap", director.getImage('font'), 'assets/img/fonts/sixcap.fnt');

                    var loadingText = new CAAT.Foundation.UI.TextActor()
                        .setFont(FIAW.Utils.getFont("sixcap"))
                        .setText(FIAW.localeText.getString("T_LOADING", "LOADING"))
                        .setTextAlign("center")
                        .setBaseline("middle")
                        .cacheAsBitmap()
                        .setScale(director.getValueForKey("IMS"), director.getValueForKey("IMS"))
                        .setLocation(director.width/2, director.height/2);

                    splashScene.addChild(loadingImg);
                    splashScene.addChild(loadingText);

                    __loadResources(director);

                }

            }

        );

    }

    /* Una vez que cargamos los assets nesesarios se sale de la splashScreen y nos dirigmos al menú principal */
    function __loadResources(director){

        var hd = director.getValueForKey("IMS")===1 ? '-hd' : '';
        loadTime = new Date();

        var web = navigator.isCocoonJS ? '' : '_web';

        director.addAudio('music', 'assets/audio/music' + web + '.mp3')
            .addAudio('snd_create_chip', 'assets/audio/snd_click.mp3')
            .addAudio('snd_drop_chip', 'assets/audio/snd_fall.mp3')
            .addAudio('snd_menu_button', 'assets/audio/menu_button.mp3')
            .addAudio('snd_no', 'assets/audio/snd_no.mp3');

        new CAAT.ImagePreloader().loadImages(
            [
                {id: 'menuS', url: 'assets/img/menu1'+ hd + '.png'},
                {id: 'menuS2', url: 'assets/img/menu2'+ hd + '.png'},
                {id: 'gameS', url: 'assets/img/game'+ hd + '.png'},
                {id: 'windowS', url: 'assets/img/windows' + hd + '.png'},
                //{id : 'sixcapC', url : 'assets/img/fonts/sixcap-colors.png'}
                {id: 'sixcap-white', url: 'assets/img/fonts/sixcap-white.png'},
                {id: 'sixcap-red', url: 'assets/img/fonts/sixcap-red.png'},
                {id: 'sixcap-black', url: 'assets/img/fonts/sixcap-black.png'},
                {id: 'sixcap-green', url: 'assets/img/fonts/sixcap-green.png'}
            ],

            function(counter, images){

                if(counter == images.length){


                    director.setImagesCache(images);

                    //__createSpriteSheet(director, "sixcapC", 'assets/img/fonts/sixcap-colors.json');
                    __createSpriteSheet(director, "menuS", 'assets/img/menu1' + hd + '.json');
                    __createSpriteSheet(director, "menuS2", 'assets/img/menu2' + hd + '.json');
                    __createSpriteSheet(director, "windowS", 'assets/img/windows' + hd + '.json');
                    __createSpriteSheet(director, "gameS", 'assets/img/game' + hd + '.json');

                    FIAW.Utils.createFont(director, "sixcap-white", director.getImage('sixcap-white'), 'assets/img/fonts/sixcap-white.fnt');
                    FIAW.Utils.createFont(director, "sixcap-red", director.getImage('sixcap-red'), 'assets/img/fonts/sixcap-red.fnt');
                    FIAW.Utils.createFont(director, "sixcap-black", director.getImage('sixcap-black'), 'assets/img/fonts/sixcap-black.fnt');
                    FIAW.Utils.createFont(director, "sixcap-green", director.getImage('sixcap-green'), 'assets/img/fonts/sixcap-green.fnt');

                    __startGame(director);

                }

            }

        );

    }

    /* Se carga el spriteSheet pasado y se mapea */
    function __createSpriteSheet(director, imageId, dataUrl){

        function cb(message, object){

            if(message == "error"){
                console.error('No se puede leer el SpriteSheet: ' + dataUrl);
            }else{
                __initializeFromTexturePackerFix(
                    director.getImage(imageId),
                    JSON.parse(object)
                );
            }

        }

        new CAAT.Module.Preloader.XHR().load(cb, dataUrl, false);

    }

    /* Se carga el archivo JSON que contiene las traducciones en FIAW.localeText*/
    function __initFiawLocale(){
        var cb = {
            localeSucces : function(){
                console.log('Locale succes.');
            },

            localeError : function(e){
                console.log('Locale error',e);
            }
        };
        console.log(navigator.language);

        FIAW.localeText = new CAAT.Module.Locale.ResourceBundle('assets/locale/texts.json', false, cb.localeSucces, cb.localeError);
    }

    /* Fix para la estructura que me da el TexturePacker, original en: CAAT.Foundation.SpriteImage().initializeFromTexturePackerJSON() */
    function __initializeFromTexturePackerFix( image, obj ) {

        for( var img in obj.sprites ) {
            var imgData= obj.sprites[img];

            var si_obj= {
                x: imgData.x,
                y: imgData.y,
                width: imgData.width,
                height: imgData.height,
                id: '0'
            };

            var si= new CAAT.Foundation.SpriteImage().initialize( image, 1, 1 );
            si.addElement(0,si_obj);
            CAAT.currentDirector.addImage( img, si );
        }
    }

    /* Se cargan todos los modulos que usará la aplicación */
    function __loadDependencies(director){

        CAAT.ModuleManager
            .debug(true)
            .baseURL('js')
            .setModulePath("FIAW.Actor", 'actor' )
            .setModulePath('FIAW.Manager', 'manager' )
            .setModulePath("FIAW.Game", "game")
            .bring(
                [
                    "FIAW.Game.Utils",
                    "FIAW.Actor.MenuScene",
                    "FIAW.Manager.GamePlayManager"
                ]
            ).onReady( function() {
                
                __initFiawLocale();
                __initUtils();
                __showSplashScreen(director);
            });
    }

    /* Eliminamos la escena de carga y añadimos el menu */
    function __startGame(director){

        menuScene = new FIAW.Actor.MenuScene(director);

        var ready = function(){
            director.emptyScenes();
            director.addScene(menuScene);
            director.setScene(0);
            director.musicPlay('music');
        };

        loadTime = new Date() - loadTime;
        console.log('Assets load time: '+ (loadTime/1000).toString() + ' sec.');
        var readyTime = (loadTime < FIAW.Globals.G_MIN_TIME_LOADING) ? FIAW.Globals.G_MIN_TIME_LOADING-loadTime : 0;

        setTimeout(function(){
            ready();
        }, readyTime);
    }

    function loadPreferences(director){
        FIAW.musicEnabled = CAAT.Module.Storage.LocalStorage.load("FIAW.musicEnabled", true );
        FIAW.soundEnabled = CAAT.Module.Storage.LocalStorage.load("FIAW.soundEnabled", true );
        FIAW.adsEnabled = true;
        director.setSoundEffectsEnabled( FIAW.soundEnabled );
        director.setMusicEnabled( FIAW.musicEnabled );
    }

    /* Iniciamos la aplicación */
    function __fiaw_init() {
        
        var director = __createCanvas();
        loadPreferences(director);
        director.enableResizeEvents(CAAT.Director.prototype.RESIZE_PROPORTIONAL, null);
        CAAT.setCoordinateClamping(false);

        __loadDependencies(director);
        CAAT.loop(60);

        CocoonJS.App.onSuspended.addEventListener( function() {

            //Desconecto al jugador si pasa la aplicación a un segundo plano.
            if(CAAT.currentDirector.currentScene.gameMode == FIAW.Globals.G_PLAYER_VS_ONLINE) {
                CAAT.currentDirector.currentScene.gamePlayManager.multiplayerManager.setDisconnect(true);
                CAAT.currentDirector.currentScene.gamePlayManager.returnMenuScene();
            }
            CAAT.endLoop();
        });

        CocoonJS.App.onActivated.addEventListener( function() {
            CAAT.loop(60);
        });


    }

    window.addEventListener('load', __fiaw_init, false);

})();