
CAAT.Module( {
    defines : "FIAW.Manager.BotManager",
    depends: [
        "FIAW.Actor.GamePlayScene"
    ],

    extendsWith : function() {

        return {

            director: null,
            gamePlayScene: null,
            gamePlayManager: null,

            thinkTime: null,
            discardCols: [],
            checkCancel: false,

            __init: function(gamePlayManager){
                this.gamePlayManager = gamePlayManager;
                this.director = this.gamePlayManager.director;
                this.gamePlayScene = this.gamePlayManager.gamePlayScene;
                return this;
            },

            cancel: function(){
                //this.timeOutMove();
                this.checkCancel = true;
            },

            columDrop: function(col){
                this.dropOn(this.gamePlayScene.coordsCols[col]);
            },

            //Transofrma unas coordenadas x en una columna
            dropOn: function(x){
                var xRnd = this.gamePlayScene.gameBoard.width/2;
                if(!this.gamePlayScene.actualChip) this.gamePlayScene.__createChip({x:xRnd}, this.gamePlayScene);

                this.gamePlayScene.__dropChip({'x':x}, this.gamePlayScene);
                
            },

            //Soltamos la ficha en una columna elegida al azar
            timeOutMove: function(){
                var colRnd;
                var loop = true;
                while(loop){
                    colRnd = Math.floor(Math.random()*(this.gamePlayScene.coordsCols.length));

                    var drop = this.gamePlayScene.boardColsMap[colRnd].lastIndexOf(false);

                    if(drop != -1) loop = false;

                }

                this.dropOn(this.gamePlayScene.coordsCols[colRnd]);
            },

            //Una comprobación rapida que nos indica si existe posibilidad de perder en un solo movimiento
            fastCheck: function(board){
                var botTurn = this.gamePlayScene.botTurn,
                    playerTurn = (botTurn == FIAW.Globals.G_PLAYER_1) ? FIAW.Globals.G_PLAYER_2 : FIAW.Globals.G_PLAYER_1;

                var moves = this.getColumMoves(board);
                var col = false;

                for(var i = 0; i < moves.length; i++){

                    if(moves[i] !== false){

                        board[i][moves[i]] =  botTurn;

                        if(this.gamePlayManager.checkWinGame(i, moves[i], botTurn, board)){
                            col = i;
                            board[i][moves[i]] =  false;
                            break;
                        }

                        board[i][moves[i]] =  playerTurn;

                        if(this.gamePlayManager.checkWinGame(i, moves[i], playerTurn, board)){
                            col = i;
                            board[i][moves[i]] =  false;
                            break;
                        }

                        board[i][moves[i]] =  false;

                    }

                }

                return col;
            },

            //Compruebo 2 jugadas por delante para tomar una desición
            lookAhead: function(board){
                var botTurn = this.gamePlayScene.botTurn,
                    playerTurn = (botTurn == FIAW.Globals.G_PLAYER_1) ? FIAW.Globals.G_PLAYER_2 : FIAW.Globals.G_PLAYER_1;

                var moves = this.getColumMoves(board);

                this.discardCols = [];

                col = this.fastCheck(board);

                if(col === false){
                    for(var i = 0; i < moves.length; i++){

                        if(moves[i] !== false){

                            board[i][moves[i]] = botTurn;

                            var humanMoves = this.getColumMoves(board);

                            for(var j = 0; j < humanMoves.length; j++){

                                if(humanMoves[j] !== false){

                                    board[j][humanMoves[j]] = playerTurn;
                                    col = this.fastCheck(board);
                                    board[j][humanMoves[j]] = false;

                                    if(col !== false){

                                        //Compruebo si la columna elegida es una de las descartadas
                                        var discard = false;
                                        if(this.discardCols.length >= 1){
                                            for(var n = 0; n < this.discardCols.length; n++){
                                                if(this.discardCols[n] === col){
                                                    discard = true;
                                                    col = false;
                                                }
                                            }
                                        }

                                        //Si no es de las descartadas compruebo que no me perjudica en la siguiente mano
                                        if(!discard){

                                            var row = board[col].lastIndexOf(false);
                                            if(row >= 1){

                                                board[col][row] = botTurn;
                                                board[col][row-1] = playerTurn;

                                                //Si me perjudica descarto la columna, si no lanzo ficha
                                                if(this.gamePlayManager.checkWinGame(col, row-1, playerTurn, board)){
                                                    board[col][row] = false;
                                                    board[col][row-1] = false;
                                                    this.discardCols.push(col);
                                                    col = false;
                                                    break;
                                                }else{
                                                    return col;
                                                }

                                            }else{
                                                return col;
                                            }


                                        }

                                    }
                                    

                                }

                            }

                            board[i][moves[i]] = false;

                        }

                    }
                }

                return col;

            },

            //Analiza el tablero para realizar el movimiento adecuado teniendo en cuenta diferentes factores
            analizeAndMove: function(){
                this.thinkTime = new Date();

                var botTurn = this.gamePlayScene.botTurn,
                    playerTurn = (botTurn == FIAW.Globals.G_PLAYER_1) ? FIAW.Globals.G_PLAYER_2 : FIAW.Globals.G_PLAYER_1;

                var board = this.gamePlayManager.getBoardState(),
                    moves = this.getColumMoves(board);

                var col = this.lookAhead(board);
                var self = this;

                //setTimeout(function(){

                        if(col === false){
                        
                            //Si no se devolvió una buena columna entonces lanzo al azar evitando las descartadas
                            if(self.discardCols.length >= 1){
                                var loop = true;

                                var goodMoves = []; //Almaceno las columnas permitidas

                                for(var n = 0; n < moves.length; n++){
                                    for(var i = 0; i  < self.discardCols.length; i++){
                                        if(self.discardCols[i] === n){
                                            moves[n] = false;
                                        }
                                    }

                                    if(moves[n] !== false) goodMoves.push(n);
                                }
                                
                                //Si hay columna permitidas lanzo al azar entre ellas
                                if(goodMoves.length >= 1){
                                    col = goodMoves[Math.floor(Math.random() * goodMoves.length)];
                                    self.columDrop(col);
                                }else{
                                    self.timeOutMove(); //Si no hay permitidas lanzo totalmente al azar, así evito bloquear el tablero aunque pierda.
                                }

                            }else{

                                //No hay descartada, lanzo totalmente al azar
                                self.timeOutMove();
                            }

                        }else{
                            self.columDrop(col); //Lanzo en la columna indicada
                        }

                        console.log(self.getThinkDate(self.thinkTime));
                //}, 1000 + (Math.random() * 1300));

            },

            getThinkDate: function(date){
                return "Bot thought in " + ((new Date() - date)/1000).toString() + " sec.";
            },

            //Devuelve un array con las filas de cada columna donde podemos soltar ficha
            getColumMoves: function(board){
                var arr = [];

                for(var i = 0; i < board.length; i++) {
                    var row = board[i].lastIndexOf(false);

                    if(row === -1) row = false;

                    arr.push(row);
                }

                return arr;
            }


        };

    }

});