
CAAT.Module( {
    defines : "FIAW.Manager.GamePlayManager",
    depends: [
        "FIAW.Actor.GamePlayScene",
        "FIAW.Manager.BotManager",
        "FIAW.Manager.MultiplayerManager"
    ],

    extendsWith : function() {

        return {

            director: null,
            menuScene: null,
            gamePlayScene: null,
            botManager: null,
            multiplayerManager: null,
            sceneAdded: false,

            arrayWinCheck: [],
            arrayWin: [],
            winType: null,

            __init: function(director, menuScene){
                this.director = director;
                this.menuScene = menuScene;
                this.initGamePlayScene();
                this.initBotManager();
                this.initMultiplayerManager();
                FIAW.Utils.initInAppsManager(this);
                return this;
            },

            initGamePlayScene: function(){
                this.gamePlayScene = new FIAW.Actor.GamePlayScene(this);
                return this;
            },

            initBotManager: function(){
                this.botManager = new FIAW.Manager.BotManager(this);
                return this;
            },

            initMultiplayerManager: function(){
                this.multiplayerManager = new FIAW.Manager.MultiplayerManager(this);
                return this;
            },

            activeGamePlay: function(gameMode){
                var self = this;
                if(typeof gameMode !== 'undefined') this.gamePlayScene.gameMode = gameMode;
                if(!this.sceneAdded){
                    this.director.addScene(this.gamePlayScene);
                    this.sceneAdded = true;
                }
                
                this.menuScene.animateSceneOut(gameMode, function(){
                    self.director.easeNone(1,0,0);
                    self.menuScene.transBackground.setAlpha(0);
                });

            },


            returnMenuScene: function(){
                var self = this;
                this.gamePlayScene.__createTransBackground();

                var behavior = new CAAT.Behavior.AlphaBehavior()
                    .setValues(0, 1)
                    .setDelayTime(0, 150)
                    .setCycle(false);

                behavior.addListener({
                    behaviorExpired : function(){
                        
                        self.gamePlayScene.resetGame();
                
                        //Elimino del contador la contabilización de partidas ganadas/perdidas
                        self.gamePlayScene.countWin = [0,0];

                        self.director.easeNone(0,1,0);
                        self.gamePlayScene.transBackground.setAlpha(0);
                    }
                });

                this.gamePlayScene.transBackground.addBehavior(behavior);

            },

            //Devuelve la escena del gamePlay a su estado inicial al reinciar o salir de ella
            resetGamePlayScene: function(){
                this.gamePlayScene.resetGame();
            },

            soundButtonPressed: function(e, self){
                FIAW.soundEnabled = !FIAW.soundEnabled;
                this.menuScene.settingSoundState.setBackgroundImage(
                    this.director.getImage( (FIAW.soundEnabled) ? 'ok' : 'off' )
                ).rescale();

                if(this.gamePlayScene.pauseWin){
                    this.gamePlayScene.pauseWin.soundStatusText.setText(
                        ((FIAW.soundEnabled) ? FIAW.localeText.getString("T_ON", "ON") : FIAW.localeText.getString("T_OFF", "OFF")) + " "
                    ).setFont(FIAW.Utils.getFont((FIAW.soundEnabled) ? "sixcap-red" : "sixcap-black"));
                }

                CAAT.Module.Storage.LocalStorage.save("FIAW.soundEnabled", FIAW.soundEnabled );
                this.director.setSoundEffectsEnabled( FIAW.soundEnabled );
            },

            musicButtonPressed: function(e, self){
                FIAW.musicEnabled = !FIAW.musicEnabled;
                this.menuScene.settingMusicState.setBackgroundImage(
                    this.director.getImage( (FIAW.musicEnabled) ? 'ok' : 'off' )
                ).rescale();

                if(this.gamePlayScene.pauseWin){
                    this.gamePlayScene.pauseWin.musicStatusText.setText(
                        ((FIAW.musicEnabled) ? FIAW.localeText.getString("T_ON", "ON") : FIAW.localeText.getString("T_OFF", "OFF")) + " "
                    ).setFont(FIAW.Utils.getFont((FIAW.musicEnabled) ? "sixcap-red" : "sixcap-black"));
                }

                CAAT.Module.Storage.LocalStorage.save("FIAW.musicEnabled", FIAW.musicEnabled );
                this.director.setMusicEnabled( FIAW.musicEnabled );

                if(FIAW.musicEnabled){
                    this.director.musicPlay('music');
                }else{
                    this.director.musicStop();
                }
            },

            //Elimono los tres botones de "remove ads"
            removeAdsButtons: function(){
                if(this.gamePlayScene.gameOverWindow){
                    if(this.gamePlayScene.gameOverWindow.removeAdsButton){
                        this.gamePlayScene.gameOverWindow.removeAdsButton.setDiscardable(true).setExpired(true);
                    }
                }

                if(this.gamePlayScene.pauseWin){
                    if(this.gamePlayScene.pauseWin.removeAdsText){
                        this.gamePlayScene.pauseWin.removeAdsText.setDiscardable(true).setExpired(true);
                        this.gamePlayScene.pauseWin.enableEvents(false);
                    }
                }

                if(this.menuScene.removeAdsText) this.menuScene.removeAdsText.setDiscardable(true).setExpired(true);
            },


            //Devuelve el estado del tablero como un array conteniendo los turnos de cada ficha
            getBoardState: function(){
                var boardState = [],
                    len = this.gamePlayScene.boardColsMap.length;

                for(var i = 0; i < len; i++){
                    var col = [];
                        for(var n = 0; n < this.gamePlayScene.boardColsMap[i].length; n++){
                            var state = (this.gamePlayScene.boardColsMap[i][n]) ? this.gamePlayScene.boardColsMap[i][n].turn : false;
                            col.push(state);
                        }
                    boardState.push(col);
                }

                return boardState;
            },

            resetArrayWin: function(){
                this.arrayWinCheck = [];
            },

            setWinChip: function(col, row){
                this.arrayWinCheck.push([col, row]);
            },

            saveArrayWin: function(winType){
                this.arrayWin = this.arrayWinCheck;
                this.winType = winType;
            },

            checkVerticalMatch: function(col, row, turn, board){
                this.resetArrayWin();

                var n = 0;

                for(var i = 0;i < FIAW.Globals.G_BOARD_ROWS;i++){

                    if(board[col][i] === turn) {
                        n++;
                        this.setWinChip(col, i);
                    }else{
                        n = 0;
                        this.resetArrayWin();
                    }

                    if(n === 4) {
                        this.saveArrayWin('vertical');
                        break;
                    }
                }

                return (n === 4) ? true : false;
            },

            checkHorizontalMatch: function(col, row, turn, board){
                this.resetArrayWin();
                var n = 0;

                for(var i = 0;i < FIAW.Globals.G_BOARD_COLS;i++){

                    if(board[i][row] === turn) {
                        n++;
                        this.setWinChip(i, row);
                    }else{
                        n = 0;
                        this.resetArrayWin();
                    }

                    if(n === 4) {
                        this.saveArrayWin('horizontal');
                        break;
                    }
                }

                return (n === 4) ? true : false;
            },

            checkDiagTopDownMatch: function(col, row, turn, board){
                this.resetArrayWin();
                var n = 0;

                var less = Math.min(col, row);

                var rowStart = row - less,
                    colStart = col - less;

                while(true){

                    if(board[colStart][rowStart] === turn){
                        n++;
                        this.setWinChip(colStart, rowStart);
                    }else{
                        n = 0;
                        this.resetArrayWin();
                    }

                    if(n === 4) {
                        this.saveArrayWin('diag-top-down');
                        break;
                    }

                    if(colStart === FIAW.Globals.G_BOARD_COLS -1 || rowStart === FIAW.Globals.G_BOARD_ROWS -1) break;

                    colStart++;
                    rowStart++;

                }

                return (n == 4) ? true : false;
            },

            checkDiagDownTopMatch: function(col, row, turn, board){
                this.resetArrayWin();
                var n = 0;

                var rowStart = (col + row > FIAW.Globals.G_BOARD_ROWS-1) ? FIAW.Globals.G_BOARD_ROWS-1 : col + row,
                    colStart = col - (rowStart - row);

                while(true){

                    if(board[colStart][rowStart] === turn){
                        n++;
                        this.setWinChip(colStart, rowStart);
                    }else{
                        n = 0;
                        this.resetArrayWin();
                    }

                    if(n === 4) {
                        this.saveArrayWin('diag-down-top');
                        break;
                    }

                    if(colStart === FIAW.Globals.G_BOARD_COLS -1 || rowStart === 0) break;

                    colStart++;
                    rowStart--;

                }

                return (n === 4) ? true : false;
            },

            checkTieGame: function(board){
                var tie = true;

                for(var i = 0; i < board.length; i++){
                    if(board[i].indexOf(false) !== -1){ tie = false; }
                }

                return tie;
            },

            checkWinGame: function(col, row, turn, board){
                var r = false;

                if(this.checkVerticalMatch(col, row, turn, board)) r = true;
                if(this.checkHorizontalMatch(col, row, turn, board)) r = true;
                if(this.checkDiagTopDownMatch(col, row, turn, board)) r = true;
                if(this.checkDiagDownTopMatch(col, row, turn, board)) r = true;

                return r;
            }

        };

    }


});