
CAAT.Module( {
    defines : "FIAW.Manager.InAppsManager",

    extendsWith : function() {

        return {

            params: {
                "managed" : true,
                "sandbox" : false
            },
            isStoreAvailable: false,
            products: null,

            director : null,
            gamePlayManager : null,

            __init: function(gamePlayManager){
                this.gamePlayManager = gamePlayManager;
                this.director = this.gamePlayManager.director;

                if( !FIAW.Utils.isAndroidOrIos()){
                    FIAW.adsEnabled = false;
                }

                if(!navigator.isCocoonJS) return this;

                this.initStoreConfig();

                return this;
            },

            initStoreConfig: function(){
                var self = this;
                if(CocoonJS.Store.nativeExtensionObjectAvailable && CocoonJS.Store.canPurchase()){

                    this.isStoreAvailable = true;

                    CocoonJS.Store.onProductsFetchStarted.addEventListener(this.onProductsFetchStarted);
                    CocoonJS.Store.onProductsFetchFailed.addEventListener(this.onProductsFetchFailed);
                    CocoonJS.Store.onProductsFetchCompleted.addEventListener(this.onProductsFetchCompleted);

                    CocoonJS.Store.onRestorePurchasesStarted.addEventListener(function(e){
                        self.onRestorePurchasesStarted(e, self);
                    });

                    CocoonJS.Store.onRestorePurchasesCompleted.addEventListener(function(e){
                        self.onRestorePurchasesCompleted(e, self);
                    });

                    CocoonJS.Store.onRestorePurchasesFailed.addEventListener(function(e){
                        self.onRestorePurchasesFailed(e, self);
                    });

                    CocoonJS.Store.onProductPurchaseStarted.addEventListener(this.onProductPurchaseStarted);
                    CocoonJS.Store.onProductPurchaseFailed.addEventListener(this.onProductPurchaseFailed);


                    CocoonJS.Store.onProductPurchaseCompleted.addEventListener(function (purchase){
                        self.onProductPurchaseCompleted(purchase, self);
                    });

                    CocoonJS.Store.requestInitialization(this.params);
                    CocoonJS.Store.start();
                    CocoonJS.Store.fetchProductsFromStore(['com.ideateca.fourinarow.adremoval']);

                    this.products = CocoonJS.Store.getProducts();
                    console.log('Productos', this.products.length);

                    this.purchases = CocoonJS.Store.getPurchases();
                    console.log(this.purchases);

                    if(this.purchases && this.purchases.length > 0){
                        FIAW.adsEnabled = false;
                    }

                }

            },

            onRestorePurchasesCompleted : function(e, self){
                console.log('onRestorePurchasesCompleted');
                var menu = self.gamePlayManager.menuScene;
                menu.loadingWindow.hideWindow();
            },

            onRestorePurchasesFailed : function(error, self){
                console.log('onRestorePurchasesFailed');
                var menu = self.gamePlayManager.menuScene;
                menu.loadingWindow.hideWindow();
                CocoonJS.App.showMessageBox(
                    "Restore Purchase",
                    "Purchase restoration failed with error: ':"+error+"'.",
                    "Ok"
                );
            },

            onRestorePurchasesStarted : function(e, self){
                console.log('onRestorePurchasesStarted');
                var menu = self.gamePlayManager.menuScene;
                menu.loadingWindow.showWindow(FIAW.localeText.getString("T_RESTORING", "RESTORING") + "... ");
            },

            onProductPurchaseCompleted: function(purchase, self){
                switch(purchase.purchaseState){
                    case CocoonJS.Store.PurchaseState.PURCHASED:
                        console.log('PURCHASED', purchase.transactionId);
                        FIAW.adsEnabled = false;
                        self.gamePlayManager.removeAdsButtons();
                        CocoonJS.Store.addPurchase(purchase);
                        FIAW.Utils.adsManager.hideBanner();
                    break;
                    case CocoonJS.Store.PurchaseState.CANCELED:
                        console.log('CANCELED', purchase.transactionId);
                        CocoonJS.Store.removePurchase(purchase.transactionId);
                    break;
                    case CocoonJS.Store.PurchaseState.REFUNDED:
                        console.log('REFUNDED', purchase.transactionId);
                        CocoonJS.Store.removePurchase(purchase.transactionId);
                    break;
                    case CocoonJS.Store.PurchaseState.EXPIRED:
                        console.log('EXPIRED', purchase.transactionId);
                        CocoonJS.Store.removePurchase(purchase.transactionId);
                    break;
                }
                CocoonJS.Store.finishPurchase(purchase.transactionId);
            },

            onProductsFetchStarted: function(){
                console.log('onProductsFetchStarted');
            },

            onProductsFetchFailed: function(err){
                console.log('onProductsFetchFailed: ', err);
            },

            onProductsFetchCompleted: function(products){
                console.log('onProductsFetchCompleted');
                console.log(JSON.stringify(products));
            },

            onProductPurchaseStarted: function(product){
                console.log('onProductPurchasesStarted: ', product);
            },

            onProductPurchaseFailed: function(product, err){
                console.log('onProductPurchaseFailed: ', product, err);
            },

            removeAds: function(){
                if(this.isStoreAvailable && this.products.length > 0){
                    var product = this.products[0];
                    CocoonJS.Store.purchaseProduct(product.productId);
                }
            },

            restorePurchases : function(){
                if(navigator.isCocoonJS){
                    console.log('Restore PURCHASED');
                    if(this.isStoreAvailable) CocoonJS.Store.restorePurchases();
                }
            }


        };

    }

});