CAAT.Module( {
    defines : "FIAW.Manager.MultiplayerManager",

    extendsWith : function() {

        return {

            gamePlayManager: null,
            socialService: null,
            multiplayerService: null,
            usingGameCenter: false,
            loopbackServices: [null, null],
            players: null,
            gamePlayScene: null,

            playAgainPlayerCount: 0,

            msgWindow: null,
            isDisconnect: null,
            score: null,

            leaderBoard: null,

            leaderBoardID: {
                ios : "scoresfourinarow",
                android : "CgkIss-vwIwJEAIQAA"
            },

            __init: function(gamePlayManager){
                this.gamePlayManager = gamePlayManager;
                console.log("Multiplayer Manager ON");

                this.initMultiplayer();
                return this;
            },

            incrementAndSaveScore: function(winner){
                if(this.socialService && this.players[winner].match !== null){
                    this.score += 1;
                    CAAT.Module.Storage.LocalStorage.save("FIAW.multiplayerWins",this.score);
                    console.log('Score local saved: ' + this.score);

                    this.socialService.submitScore(this.score, function(error) {
                        if (error) {
                            console.error("Error submitting score: " + error);
                        } else {
                            console.log("Score submitted!");
                        }
                    }, this.leaderBoard);
                }
            },

            initMultiplayer: function(){
                var self = this;

                this.score = CAAT.Module.Storage.LocalStorage.load("FIAW.multiplayerWins",0);
                console.log('Score now: ', this.score);

                if(CocoonJS.Social.GameCenter.nativeExtensionObjectAvailable){
                    var gc = CocoonJS.Social.GameCenter;
                    this.socialService = gc.getSocialInterface();
                    this.multiplayerService = gc.getMultiplayerInterface();
                    this.usingGameCenter = true;
                    this.leaderBoard = new CocoonJS.Social.ScoreParams(null, this.leaderBoardID.ios);

                }else if(CocoonJS.Social.GooglePlayGames.nativeExtensionObjectAvailable){
                    var gp = CocoonJS.Social.GooglePlayGames;
                    gp.init({defaultLeaderboard: this.leaderBoardID.android });
                    this.socialService = gp.getSocialInterface();
                    this.multiplayerService = gp.getMultiplayerInterface();
                }

                this.loopbackServices = [
                    new CocoonJS.Multiplayer.LoopbackService(),
                    new CocoonJS.Multiplayer.LoopbackService()
                ];

                if(this.multiplayerService){

                    this.multiplayerService.onInvitationReceived.addEventListener(function(){

                        console.log("Invitación recibida");

                    });

                    this.multiplayerService.onInvitationLoaded.addEventListener(function (match, error){

                        console.log("Invitación lista: (Error: " + JSON.stringify(error) + ")");
                        self.__handleMatch(match, error, self);

                    });

                }
                
            },


            initMultiplayerGame: function(players, services){

                //Iniciamos la escena y el modo de juego online
                this.gamePlayManager.activeGamePlay(FIAW.Globals.G_PLAYER_VS_ONLINE);
                this.gamePlayScene = this.gamePlayManager.gamePlayScene;

                this.players = players.slice();

                this.players.sort(function (a,b){
                    return a.userID < b.userID ? -1 : 1;
                });

                //Añadimos la instancia de match correspondiente a cada jugador
                for(var i = 0; i < this.players.length; ++i){
                    this.players[i].match = null;
                    for(var j = 0; j < services.length; ++j){
                        if (services[j] && services[j].getMatch().getLocalPlayerID() === this.players[i].userID ) {
                            this.players[i].match = services[j].getMatch();
                        }
                    }

                }

                //console.log('matchs: ', JSON.stringify(services));

                //Se decide al azar que jugador comienza el juego
                if(this.players[0].match !== null){
                    var gameTurn = Math.random() < 0.5 ? 0 : 1;
                    this.players[0].match.sendDataToAllPlayers(JSON.stringify(["setTurn", gameTurn]));
                }
            },

            setDisconnect: function(msg){
                this.isDisconnect = true;
                for(var i = 0; i < this.players.length; i++){
                    if(this.players[i].match !== null){
                        if(msg){
                            this.players[i].match.sendDataToAllPlayers(JSON.stringify("exit"));
                            this.players[i].match.disconnect();
                        }
                    }
                }

                this.gamePlayScene.stopTime();
            },

            __handleMatch: function(match, error, self){

                var requestPlayersCallback = function(players, error){

                    if(error){
                        console.log('RequestPlayersCallback Error: ', error.message);
                        return;
                    }


                    if(self.isDisconnect === true) return;

                    if(players.length != 2){
                        self.setDisconnect(true);
                        self.disconnectedPlayerWindow();
                        return;
                    }


                    self.initMultiplayerGame(players, [self.multiplayerService]);
                    match.start();


                };

                if(match){

                    //COmprobamos al cambiar el estado si todos los jugadores se han conectado para iniciar la partida.
                    match.onMatchStateChanged.addEventListener(function (match, player, state){
                        if(state == CocoonJS.Multiplayer.ConnectionState.DISCONNECTED){
                            self.setDisconnect(false);
                            self.disconnectedPlayerWindow();
                        }

                        console.log('MatchChanged', player, state);

                        if(match.getExpectedPlayerCount() === 0){
                            match.requestPlayersInfo(requestPlayersCallback);
                        }

                    });


                    match.onMatchDataReceived.addEventListener(function (match, data, playerID){
                        var msg = JSON.parse(data);

                        switch(msg[0]){
                            case "setTurn":
                                self.firstTurnMessage(msg[1]);
                            break;
                            case "dropOn":
                                if(playerID != self.gamePlayScene.multiplayerData.localID){
                                    self.dropChipOn(msg[1], true);
                                }
                            break;
                            case "playAgain":
                                self.playAgain();
                            break;
                            case "exit":
                                self.disconnect(false);
                                self.disconnectedPlayerWindow();
                            break;
                        }


                    });

                    /* match.onMatchConnectionWithPlayerFailed.addEventListener(function (match, player, error){
                        self.setDisconnect(false);
                    });

                    match.onMatchFailed.addEventListener(function (match, error){
                        self.setDisconnect(false);
                    }); */

                    if(match.getExpectedPlayerCount() === 0){
                        match.requestPlayersInfo(requestPlayersCallback);
                    }

                }else{
                    if(!error){
                        console.log('Multijugador cancelado');
                    }else{
                        console.log('ERROR: ' + error.message);
                    }
                }
            },

            //Cuando llegan dos peticiones de playAgain reinciamos la escena
            playAgain: function(){
                this.playAgainPlayerCount++;

                if(this.playAgainPlayerCount == 2){
                    
                    if(this.msgWindow){
                        var self = this;
                        this.msgWindow.setDiscardable(true).setExpired(true);
                    }
                    this.gamePlayScene.__initialize(true);

                    this.playAgainPlayerCount = 0;

                }
            },

            waitForPlayerWindow: function(){
                if(this.gamePlayScene.gameOverWindow) this.gamePlayScene.gameOverWindow.setDiscardable(true).setExpired(true);
                if(this.msgWindow) this.msgWindow.setDiscardable(true).setExpired(true);

                this.msgWindow = new FIAW.Actor.GameOverWindow(this.gamePlayScene, 2);
                this.gamePlayScene.addChild(this.msgWindow);
            },

            disconnectedPlayerWindow: function(){
                if(this.gamePlayScene.gameOverWindow) this.gamePlayScene.gameOverWindow.setDiscardable(true).setExpired(true);
                if(this.msgWindow) this.msgWindow.setDiscardable(true).setExpired(true);

                this.msgWindow = new FIAW.Actor.GameOverWindow(this.gamePlayScene, 3);
                this.gamePlayScene.addChild(this.msgWindow);
            },

            timeOutMove: function(){
                var colRnd;
                var loop = true;
                while(loop){
                    colRnd = Math.floor(Math.random()*(this.gamePlayScene.coordsCols.length));

                    var drop = this.gamePlayScene.boardColsMap[colRnd].lastIndexOf(false);

                    if(drop != -1) loop = false;

                }
                this.dropChipOn(colRnd, false);
            },

            dropChipOn: function(col, noMulti){
                var x = this.gamePlayScene.coordsCols[col];

                var xRnd = this.gamePlayScene.gameBoard.width/2;
                if(!this.gamePlayScene.actualChip) this.gamePlayScene.__createChip({x:xRnd}, this.gamePlayScene);

                this.gamePlayScene.__dropChip({'x':x}, this.gamePlayScene, noMulti); //El true evita que se envie de nuevo
            },

            firstTurnMessage: function(turn){
                this.gamePlayScene.multiplayerData.firstTurn = turn;
                this.gamePlayScene.multiplayerData.players = [this.getPlayerID(0), this.getPlayerID(1)];
                this.gamePlayScene.multiplayerData.localID = (this.players[0].match !== null) ? this.getPlayerID(0) : this.getPlayerID(1);

                var other = (turn === 0) ? 1 : 0;
                this.gamePlayScene.multiplayerData.playersName[turn] = this.getPlayerAlias(turn);
                this.gamePlayScene.multiplayerData.playersName[other] = this.getPlayerAlias(other);

            },

            getPlayerAlias: function(index) {
                return this.players[index].userName;
            },

            getPlayerID: function(index){
                return this.players[index].userID;
            },

            isLocalTurn: function(turn){
                return (this.players[turn].match !== null) ? true : false;
            },

            startMultiplayerGame: function(){
                if(FIAW.Utils.isAndroidOrIos() && !FIAW.Utils.isAmazonApp()){
                    var self = this;
                    if(!this.socialService.isLoggedIn()){

                        this.socialService.login(function (loggedIn, error){

                            if(loggedIn){
                                self.findNewMatch();
                            }else{
                                console.log('ERROR: ' + JSON.stringify(error));
                            }

                        });
                    }else{
                        this.findNewMatch();
                    }
                }

            },

            showRanking: function(){
                if(FIAW.Utils.isAndroidOrIos() && !FIAW.Utils.isAmazonApp()){
                    var self = this;
                    if(!this.socialService.isLoggedIn()){

                        this.socialService.login(function (loggedIn, error){

                            if(loggedIn){
                                self.socialService.showLeaderboard();
                            }else{
                                console.log('ERROR: ' + JSON.stringify(error));
                            }

                        });
                    }else{
                        self.socialService.showLeaderboard();
                    }
                }
            },

            findNewMatch: function(cb){
                var request = CocoonJS.Multiplayer.MatchRequest(2,2); //Min&Max 2 Players
                this.isDisconnect = false;
                var self = this;
                this.multiplayerService.findMatch(request, function(match, error){
                    self.__handleMatch(match, error, self);
                });
            }

        };

    }

});